import os
import random

from refinitiv.data import _configure as configure
from . import conftest
from ..conftest import remove_project_config

BASE_URL = "https://api.refinitiv.com"
TEST_BASE_URL = "https://api.test.refinitiv.com"

EXPECTED_CONFIG_FILE_PATHS_COUNT = 2

# endregion


# region USER_CONFIG_FILE
def test_user_config_file_path_not_exists(user_config_path, write_user_config):
    path = write_user_config('{"prop": "value"}')

    configure.reload()
    assert EXPECTED_CONFIG_FILE_PATHS_COUNT == len(configure._config_files_paths)
    assert path == configure._config_files_paths[1]

    conftest.remove_user_config()

    configure.reload()
    assert EXPECTED_CONFIG_FILE_PATHS_COUNT == len(configure._config_files_paths)
    assert path != configure._config_files_paths[0]


def test_user_config_file_path_already_exists(write_user_config):
    path = write_user_config('{"prop": "value"}')

    configure.reload()
    assert EXPECTED_CONFIG_FILE_PATHS_COUNT == len(configure._config_files_paths)
    assert path == configure._config_files_paths[1]
    assert path != configure._config_files_paths[0]


def test_user_config_file_path_created_after_load_package(user_config_path):
    conftest.remove_user_config()

    configure.reload()
    with open(user_config_path, "w"):
        pass
    assert EXPECTED_CONFIG_FILE_PATHS_COUNT == len(configure._config_files_paths)
    assert user_config_path != configure._config_files_paths[0]

    conftest.remove_user_config()


# endregion


# region PROJECT_CONFIG_FILE
def test_project_config_file_path_not_exists(project_config_path):
    remove_project_config()

    configure.reload()
    assert EXPECTED_CONFIG_FILE_PATHS_COUNT == len(configure._config_files_paths)
    assert project_config_path != configure._config_files_paths[1]


def test_project_config_file_path_already_exists(write_project_config):
    path = write_project_config('{"prop":"value"}')

    configure.reload()
    assert EXPECTED_CONFIG_FILE_PATHS_COUNT == len(configure._config_files_paths)
    assert path == configure._config_files_paths[0]
    assert path != configure._config_files_paths[1]


def test_project_config_file_path_created_at_runtime(project_config_path):
    remove_project_config()

    configure.reload()
    with open(project_config_path, "w"):
        pass
    assert EXPECTED_CONFIG_FILE_PATHS_COUNT == len(configure._config_files_paths)
    assert project_config_path != configure._config_files_paths[1]


# endregion


def test_environ_env_name_and_env_dir(monkeypatch, tmpdir):
    env_dir = str(random.randint(0, 100))
    d = tmpdir / env_dir
    d.mkdir()
    f = tmpdir / env_dir / (configure._default_config_file_name)
    f.write('{"prop": "value"}')

    monkeypatch.setenv(configure._RDPLIB_ENV_DIR, str(d))

    configure.reload()

    assert d == configure._project_config_dir
    assert EXPECTED_CONFIG_FILE_PATHS_COUNT == len(configure._config_files_paths)
    assert f == configure._config_files_paths[0]


def test_environ_env_name(monkeypatch):
    from pathlib import Path

    f = Path(os.getcwd()) / (configure._default_config_file_name)
    f.write_text('{"prop": "value"}', encoding="utf-8")

    monkeypatch.delenv(configure._RDPLIB_ENV_DIR, raising=False)

    configure.reload()
    assert os.getcwd() == configure._project_config_dir
    assert EXPECTED_CONFIG_FILE_PATHS_COUNT == len(configure._config_files_paths)
    assert str(f) == configure._config_files_paths[0]

    f.unlink()
    monkeypatch.delenv(configure._RD_LIB_CONFIG_PATH, raising=False)


def test_environ_env_dir(monkeypatch, tmpdir):
    configure.reload()
    env_dir = str(random.randint(0, 100))
    d = tmpdir / env_dir
    d.mkdir()
    f = tmpdir / env_dir / configure._default_config_file_name
    f.write('{"prop": "value"}')

    monkeypatch.setenv(configure._RDPLIB_ENV_DIR, str(d))

    configure.reload()
    assert d == configure._project_config_dir
    assert EXPECTED_CONFIG_FILE_PATHS_COUNT == len(configure._config_files_paths)
    assert str(f) == configure._config_files_paths[0]


def test_environ_empty(monkeypatch):
    from pathlib import Path

    f = Path(os.getcwd()) / configure._default_config_file_name
    f.write_text('{"prop": "value"}', encoding="utf-8")

    monkeypatch.delenv(configure._RDPLIB_ENV_DIR, raising=False)

    configure.reload()
    assert os.getcwd() == configure._project_config_dir
    assert EXPECTED_CONFIG_FILE_PATHS_COUNT == len(configure._config_files_paths)
    assert str(f) == configure._config_files_paths[0]

    f.unlink()


def test_config_provided_by_user(monkeypatch, tmp_path):
    assert (
        configure.get_config().get_str("sessions.platform.default.base-url")
        == "https://api.refinitiv.com"
    )
    config_path = os.path.dirname(os.path.realpath(__file__))
    monkeypatch.setenv(configure._RD_LIB_CONFIG_PATH, config_path)
    configure.reload()
    assert (
        configure.get_config().get_str("sessions.platform.default.base-url")
        == "https://api.test.refinitiv.com"
    )
    monkeypatch.delenv(configure._RD_LIB_CONFIG_PATH, raising=False)
    configure.reload()


def test_config_provided_by_user_load():
    assert (
        configure.get_config().get_str("sessions.platform.default.base-url")
        == "https://api.refinitiv.com"
    )
    config_path = os.path.dirname(os.path.realpath(__file__))
    custom_config_path = os.path.join(config_path, "refinitiv-data.config.json")
    configure.get_config().load(custom_config_path)
    assert (
        configure.get_config().get_str("sessions.platform.default.base-url")
        == "https://api.test.refinitiv.com"
    )
    configure.reload()



import allure
import pytest

from refinitiv.data.content.ownership._enums import SortOrder
from refinitiv.data.content.ownership.consolidated import recent_activity
from tests.integration.constants_list import HttpStatusCode, HttpReason
from tests.integration.content.ownership.helpers import (
    check_response_dataframe_instrument_contains_expected_universes,
    get_expected_column,
)
from tests.integration.helpers import (
    get_async_response_from_definitions,
    check_response_status,
    check_non_empty_response_data,
    check_response_dataframe_contains_columns_names,
)

EXPECTED_COLUMN_NAMES = [
    "instrument",
    "investorid",
    "investorname",
    "TR.NetSharesHeldChange",
    "TR.NetSharesHeldValChg",
]

EXPECTED_COLUMN_TITLES = [
    "Instrument",
    "Investor Id",
    "Equity owner name",
    "SharesHeld Change",
    "SharesHeld Value Change",
]


@allure.suite("Content object - Ownership Consolidated Recent Activity")
@allure.feature("Content object - Ownership Consolidated Recent Activity")
@allure.severity(allure.severity_level.CRITICAL)
class TestConsolidatedRecentActivity:
    @allure.title(
        "Create Consolidated Recent Activity definition object with valid required params and get data"
    )
    @pytest.mark.caseid("37556949")
    @pytest.mark.parametrize(
        "universe,sort_order,use_field_names_in_headers",
        [
            ("US02079K1079", "desc", False),
            (["TRI.N", "ONEX.CCP", "AAPL.O"], SortOrder.ASCENDING, True),
            (["ONEX.CCP", "invalid universe"], SortOrder.DESCENDING, False),
        ],
    )
    def test_create_consolidated_recent_activity_object_with_valid_fields_and_get_data(
        self, open_platform_session, universe, sort_order, use_field_names_in_headers
    ):
        expected_column = get_expected_column(
            use_field_names_in_headers, EXPECTED_COLUMN_TITLES, EXPECTED_COLUMN_NAMES
        )
        response = recent_activity.Definition(
            universe, sort_order, use_field_names_in_headers
        ).get_data()
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)
        check_response_dataframe_contains_columns_names(response, expected_column)
        check_response_dataframe_instrument_contains_expected_universes(
            response, universe
        )

    @allure.title(
        "Create Consolidated Recent Activity definition object with mix valid and invalid params and get data asynchronously"
    )
    @pytest.mark.caseid("37556950")
    @pytest.mark.parametrize(
        "universe,invalid_universe,sort_order",
        [
            (
                ["ONEX.CCP", "invalid universe"],
                "invalid universe",
                SortOrder.DESCENDING,
            ),
        ],
    )
    @pytest.mark.smoke
    async def test_create_consolidated_recent_activity_object_with_mix_valid_and_invalid_params_and_get_data_async(
        self, open_platform_session_async, universe, invalid_universe, sort_order
    ):
        valid_response, invalid_response = await get_async_response_from_definitions(
            recent_activity.Definition(
                universe, sort_order, use_field_names_in_headers=True
            ),
            recent_activity.Definition(
                invalid_universe, sort_order, use_field_names_in_headers=True
            ),
        )
        check_response_status(
            response=valid_response,
            expected_status_code=HttpStatusCode.TWO_HUNDRED,
            expected_http_reason=HttpReason.OK,
            expected_error_code=None,
            expected_error_message="Failed to resolve identifiers ['invalid universe']",
        )
        check_non_empty_response_data(valid_response)
        check_response_status(
            response=invalid_response,
            expected_status_code=HttpStatusCode.TWO_HUNDRED,
            expected_http_reason=HttpReason.OK,
            expected_error_code=HttpStatusCode.FOUR_HUNDRED_TWELVE,
            expected_error_message="Unable to resolve all requested identifiers. Requested items: ['invalid universe']",
        )
        check_response_dataframe_contains_columns_names(
            valid_response, EXPECTED_COLUMN_NAMES
        )
        check_response_dataframe_instrument_contains_expected_universes(
            valid_response, universe
        )



# coding: utf8

from enum import Enum, unique


@unique
class BarrierType(Enum):
    KNOCK_IN = "KnockIn"
    KNOCK_IN_KNOCK_OUT = "KnockInKnockOut"
    KNOCK_OUT = "KnockOut"
    KNOCK_OUT_KNOCK_IN = "KnockOutKnockIn"



from inspect import signature

import pytest

from refinitiv.data.content.ipa._enums import *
from refinitiv.data.content.ipa._models import *
import tests.unit.conftest as conftest


@pytest.mark.parametrize(
    argnames="input_data",
    ids=[
        "AmericanMonteCarloParameters",
        "AmortizationItem",
        "BasketItem",
        "BidAskMid",
        "DayWeight",
        "InterpolationWeight",
        "NumericalMethod",
        "PdeParameters",
    ],
    argvalues=[
        (
            AmericanMonteCarloParameters,
            {
                "american_monte_carlo_method": AmericanMonteCarloMethod.ANDERSEN,
                "additional_points": 2,
                "all_the_time_points_per_year": 3,
                "iteration_number": 4,
            },
        ),
        (
            AmortizationItem,
            {
                "start_date": "start_date",
                "end_date": "end_date",
                "amortization_frequency": AmortizationFrequency.EVERY2ND_COUPON,
                "amortization_type": AmortizationType.LINEAR,
                "remaining_notional": 5.5,
                "amount": 7.7,
            },
        ),
        (
            BasketItem,
            {
                "instrument_code": "instrument_code",
                "currency": "currency",
            },
        ),
        (
            BidAskMid,
            {
                "bid": 1.1,
                "ask": 2.2,
                "mid": 3.3,
            },
        ),
        (
            DayWeight,
            {
                "date": "date",
                "weight": 2.2,
            },
        ),
        (
            InterpolationWeight,
            {
                "days_list": [DayWeight()],
                "holidays": 2.2,
                "week_days": 3.3,
                "week_ends": 4.4,
            },
        ),
        (
            NumericalMethod,
            {
                "american_monte_carlo_parameters": AmericanMonteCarloParameters(),
                "method": Method.ANALYTIC,
                "pde_parameters": PdeParameters(),
            },
        ),
        (
            PdeParameters,
            {
                "pde_space_step_number": 1,
                "pde_standard_deviation": 2,
                "pde_time_step_number": 3,
            },
        ),
    ],
)
def test_parameter(input_data):
    cls, kwargs = input_data
    args_names = list(kwargs.keys())
    inst = cls(**kwargs)

    s = signature(cls.__init__)
    assert len(s.parameters) == (len(args_names) + 1)  # +1 for (self)

    assert conftest.has_property_names_in_class(cls, args_names), set(args_names) - set(
        conftest.get_property_names(cls)
    )

    for k, v in kwargs.items():
        attr = getattr(inst, k)
        assert attr == v, k



from enum import Enum, unique


@unique
class CurveSubType(Enum):
    BOND_CARRY = "BondCarry"
    BREAKEVEN_INFLATION_CURVE = "BreakevenInflationCurve"
    CDS_CREDIT_INDEX = "CDSCreditIndex"
    CAP_FLOOR_VOLATILITY = "CapFloorVolatility"
    CENTRAL_BANK_INTEREST_RATE_PROBABILITY = "CentralBankInterestRateProbability"
    COMMERCIAL_PAPER_BENCHMARK = "CommercialPaperBenchmark"
    CORPORATE_BOND_BENCHMARK = "CorporateBondBenchmark"
    CORPORATE_BOND_PAR = "CorporateBondPar"
    CORPORATE_BOND_SPREAD = "CorporateBondSpread"
    CORPORATE_BOND_ZERO = "CorporateBondZero"
    CORPORATE_CDS_CREDIT = "CorporateCDSCredit"
    CORPORATE_CASH_CREDIT = "CorporateCashCredit"
    CORPORATE_ISSUER = "CorporateIssuer"
    COVERED = "Covered"
    DEPOSIT = "Deposit"
    FORWARD_GOVERNMENT_BOND_ZERO = "ForwardGovernmentBondZero"
    FORWARD_GOVERNMENT_PAR = "ForwardGovernmentPar"
    FORWARD_INFLATION = "ForwardInflation"
    FORWARD_RATE_AGREEMENT_ZERO = "ForwardRateAgreementZero"
    FORWARD_STARTING_SWAP = "ForwardStartingSwap"
    GOVERNMENT_BENCHMARK = "GovernmentBenchmark"
    GOVERNMENT_BOND_BENCHMARK = "GovernmentBondBenchmark"
    GOVERNMENT_BOND_VOLATILITY = "GovernmentBondVolatility"
    GOVERNMENT_CDS_CREDIT = "GovernmentCDSCredit"
    GOVERNMENT_CASH_CREDIT = "GovernmentCashCredit"
    GOVERNMENT_STRIP_BENCHMARK = "GovernmentStripBenchmark"
    INFLATION_LINKED_BENCHMARK = "InflationLinkedBenchmark"
    INFLATION_LINKED_ZERO = "InflationLinkedZero"
    INTERBANK_OFFER_RATE = "InterbankOfferRate"
    INTEREST_RATE_SWAP = "InterestRateSwap"
    INTEREST_RATE_VOLATILITY = "InterestRateVolatility"
    MUNICIPAL_BENCHMARK = "MunicipalBenchmark"
    OVERNIGHT_INDEX_SWAP = "OvernightIndexSwap"
    OVERNIGHT_INDEX_SWAP_ZERO = "OvernightIndexSwapZero"
    SEMI_SOVEREIGN_CASH_CREDIT = "SemiSovereignCashCredit"
    SHORT_TERM_INTEREST_RATE_FUTURES_ZERO = "ShortTermInterestRateFuturesZero"
    SOLVENCY_II_CORPORATE_CREDIT_SPRD_RATING = "SolvencyIICorporateCreditSprdRating"
    SOLVENCY_II_COVERED_AND_PFANDBRIEF_ISSUER = "SolvencyIICovered&PfandbriefIssuer"
    SOLVENCY_II_FINANCIAL_CREDIT_SPRD_RATING = "SolvencyIIFinancialCreditSprdRating"
    SOLVENCY_II_INTEREST_RATE_SWAP_YIELD = "SolvencyIIInterestRateSwapYield"
    SOLVENCY_II_LIQUIDITY_SPREAD = "SolvencyIILiquiditySpread"
    SOLVENCY_II_PFANDBRIEF_YIELD_BY_CURRENCY = "SolvencyIIPfandbriefYieldByCurrency"
    SOLVENCY_II_STRUCTURED_ABS_RATING = "SolvencyIIStructuredABSRating"
    SOLVENCY_II_STRUCTURED_MBS_RATING = "SolvencyIIStructuredMBSRating"
    SOLVENCY_II_SWAP_DERIVED_ZERO = "SolvencyIISwapDerivedZero"
    SOLVENCY_II_TREASURY_VERSUS_IRS_SPREAD = "SolvencyIITreasuryVersusIRSSpread"
    SOVEREIGN_AGENCY_BENCHMARK = "SovereignAgencyBenchmark"
    SWAP_CARRY = "SwapCarry"
    SWAP_PAR = "SwapPar"
    SWAP_ZERO = "SwapZero"
    SWAPTION_VOLATILITY = "SwaptionVolatility"
    TREASURY_SPREAD = "TreasurySpread"



import allure
import pytest

import refinitiv.data as rd
from refinitiv.data.delivery import cfs
from refinitiv.data.errors import RDError
from tests.integration.constants_list import HttpStatusCode, HttpReason
from tests.integration.delivery.cfs.conftest import (
    check_amount_of_objects_in_response_not_bigger_than,
    check_no_objects_in_response,
)
from tests.integration.delivery.cfs.file_sets.conftest import (
    prepare_random_fileset_name,
    check_fileset_contains_all_attributes,
    prepare_random_package_id,
    check_fileset_contains_package_id,
    check_fileset_contains_status,
    check_filesets_in_response_available_after_date,
    check_filesets_in_response_available_before_date,
    check_filesets_in_response_have_content_from_param_after_date,
    check_filesets_in_response_have_content_to_param_before_date,
    check_filesets_in_response_created_after_date,
    check_filesets_in_response_modified_after_date,
    check_filesets_in_two_responses_are_not_the_same,
)
from tests.integration.helpers import (
    check_response_status,
    check_non_empty_response_data,
    get_async_response_from_definition,
)


@allure.suite("Delivery object - CFS file-sets")
@allure.feature("Delivery object - CFS file-sets")
@allure.severity(allure.severity_level.CRITICAL)
class TestCfsFileSets:
    bucket_name = "bulk-ESG"

    @allure.title(
        "Get filesets with valid bucket name and default amount 25 - synchronously "
    )
    @pytest.mark.caseid("36652910")
    def test_get_filesets_with_valid_bucket_name_synchronously(
        self, open_platform_session
    ):
        response = cfs.file_sets.Definition(bucket=self.bucket_name).get_data()
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)
        check_amount_of_objects_in_response_not_bigger_than(25, response)

    @allure.title(
        "Get filesets with valid bucket name and default amount 25 - asynchronously "
    )
    @pytest.mark.caseid("36652911")
    @pytest.mark.smoke
    async def test_get_filesets_with_valid_bucket_name_asynchronously(
        self, open_platform_session_async
    ):
        response = await get_async_response_from_definition(
            cfs.file_sets.Definition(bucket=self.bucket_name)
        )

        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)
        check_amount_of_objects_in_response_not_bigger_than(25, response)

    @allure.title("Get filesets with valid bucket name and closed session")
    @pytest.mark.caseid("36652912")
    def test_get_filesets_with_closed_session(self):
        session = rd.session.desktop.Definition(app_key=" ").get_session()
        rd.session.set_default(session)
        file_sets_definition = cfs.file_sets.Definition(bucket=self.bucket_name)
        with pytest.raises(
            ValueError, match="Session is not opened. Can't send any request"
        ):
            file_sets_definition.get_data()

    @allure.title("Get filesets with valid name parameter")
    @pytest.mark.caseid("36652947")
    def test_get_filesets_with_valid_name_parameter(self, open_platform_session):
        name = prepare_random_fileset_name(self.bucket_name)
        response = cfs.file_sets.Definition(
            bucket=self.bucket_name, name=name
        ).get_data()
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)
        check_amount_of_objects_in_response_not_bigger_than(1, response)

    @allure.title("Get filesets with non-existing bucket name")
    @pytest.mark.caseid("36652948")
    def test_get_filesets_with_non_existing_bucket_name(self, open_platform_session):
        with pytest.raises(RDError, match="Error code 404 | No resource found for:"):
            cfs.file_sets.Definition(bucket="non-existing bucket name").get_data()

    @allure.title(
        "Get filesets with non-existing name/list of attributes/package id/status"
    )
    @pytest.mark.caseid("36652946")
    @pytest.mark.parametrize(
        "name,attributes,package_id,status",
        [
            (
                "no name",
                {"non-existing-attribute": "some value"},
                "invalid id",
                "invalid status",
            )
        ],
    )
    def test_get_filesets_with_non_existing_params(
        self, open_platform_session, name, attributes, package_id, status
    ):
        response = cfs.file_sets.Definition(
            bucket=self.bucket_name,
            name=name,
            attributes=attributes,
            package_id=package_id,
            status=status,
        ).get_data()
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_no_objects_in_response(response)

    @allure.title("Get filesets with list of attributes which exist in fileset")
    @pytest.mark.caseid("36652949")
    @pytest.mark.parametrize("attributes", [{"ContentType": "ESG Sources"}])
    def test_get_filesets_with_existing_attributes_param(
        self, open_platform_session, attributes
    ):
        response = cfs.file_sets.Definition(
            bucket=self.bucket_name, attributes=attributes
        ).get_data()

        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)
        for fileset in response.data.raw["value"]:
            check_fileset_contains_all_attributes(fileset, attributes)

    @allure.title("Get filesets with valid package_id parameter ")
    @pytest.mark.caseid("36652955")
    def test_get_filesets_with_valid_package_id_param(self, open_platform_session):
        package_id = prepare_random_package_id(self.bucket_name)
        response = cfs.file_sets.Definition(
            bucket=self.bucket_name, package_id=package_id
        ).get_data()

        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)
        for fileset in response.data.raw["value"]:
            check_fileset_contains_package_id(fileset, package_id)

    @allure.title("Get filesets with valid status parameter ")
    @pytest.mark.caseid("36652956")
    @pytest.mark.parametrize("status", ["READY"])
    def test_get_filesets_with_valid_status_param(self, open_platform_session, status):
        response = cfs.file_sets.Definition(
            bucket=self.bucket_name, status=status
        ).get_data()
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)
        for fileset in response.data.raw["value"]:
            check_fileset_contains_status(fileset, status)

    @allure.title("Get filesets with available_from and available_to params")
    @pytest.mark.caseid("36652957")
    @pytest.mark.parametrize(
        "available_from,available_to",
        [("2020-10-23T00:00:00Z", "2022-10-23T00:00:00Z")],
    )
    def test_get_filesets_with_available_from_and_available_to_params(
        self, open_platform_session, available_from, available_to
    ):
        response = cfs.file_sets.Definition(
            bucket=self.bucket_name,
            available_from=available_from,
            available_to=available_to,
        ).get_data()
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)
        check_filesets_in_response_available_after_date(response, available_from)
        check_filesets_in_response_available_before_date(response, available_to)

    @allure.title("Get filesets with content_from and content_to params ")
    @pytest.mark.caseid("36652959")
    @pytest.mark.parametrize(
        "content_from,content_to", [("2020-10-23T00:00:00Z", "2022-10-23T00:00:00Z")]
    )
    def test_get_filesets_with_content_from_and_content_to_params(
        self, open_platform_session, content_from, content_to
    ):
        response = cfs.file_sets.Definition(
            bucket=self.bucket_name, content_from=content_from, content_to=content_to
        ).get_data()
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)
        check_filesets_in_response_have_content_from_param_after_date(
            response, content_from
        )
        check_filesets_in_response_have_content_to_param_before_date(
            response, content_to
        )

    @allure.title("Get filesets with created_since param")
    @pytest.mark.caseid("36652962")
    @pytest.mark.parametrize("created_since", ["2021-03-31T14:00:00Z"])
    def test_get_filesets_with_created_since_param(
        self, open_platform_session, created_since
    ):
        response = cfs.file_sets.Definition(
            bucket=self.bucket_name, created_since=created_since
        ).get_data()
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)
        check_filesets_in_response_created_after_date(response, created_since)

    @allure.title("Get filesets with modified_since param")
    @pytest.mark.caseid("36652963")
    @pytest.mark.parametrize("modified_since", ["2021-07-06T14:00:00Z"])
    def test_get_filesets_with_modified_since_param(
        self, open_platform_session, modified_since
    ):
        response = cfs.file_sets.Definition(
            bucket=self.bucket_name, modified_since=modified_since
        ).get_data()
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)
        check_filesets_in_response_modified_after_date(response, modified_since)

    @allure.title("Get filesets with valid skip_token parameter")
    @pytest.mark.caseid("36652964")
    def test_get_filesets_with_valid_skip_token_param(self, open_platform_session):
        response1 = cfs.file_sets.Definition(bucket=self.bucket_name, page_size=20).get_data()
        skip_token = response1.data.raw["skip_token"]
        response2 = cfs.file_sets.Definition(
            bucket=self.bucket_name, skip_token=skip_token
        ).get_data()
        check_response_status(response2, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response2)
        check_filesets_in_two_responses_are_not_the_same(response1, response2)

    @allure.title("Get filesets with valid page_size param")
    @pytest.mark.caseid("36652965")
    @pytest.mark.parametrize("page_size", [27, 100])
    def test_get_filesets_with_page_size_param(self, open_platform_session, page_size):
        response = cfs.file_sets.Definition(
            bucket=self.bucket_name, page_size=page_size
        ).get_data()
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)
        filesets_count = len(response.data.raw["value"])
        assert (
            filesets_count <= page_size
        ), f"Filesets count received {filesets_count} more than expected: {page_size}"



from tests.integration.helpers import compare_list

FCHI_CONSTITUENTS = [
    "AIRP.PA",
    "AIR.PA",
    "ALSO.PA",
    "MT.AS",
    "AXAF.PA",
    "BNPP.PA",
    "BOUY.PA",
    "CAPP.PA",
    "CARR.PA",
    "CAGR.PA",
    "DANO.PA",
    "DAST.PA",
    "ENGIE.PA",
    "ESLX.PA",
    "EUFI.PA",
    "HRMS.PA",
    "PRTP.PA",
    "OREP.PA",
    "LVMH.PA",
    "LEGD.PA",
    "MICP.PA",
    "ORAN.PA",
    "PERP.PA",
    "PUBP.PA",
    "RENA.PA",
    "SAF.PA",
    "SGOB.PA",
    "SASY.PA",
    "SCHN.PA",
    "SOGN.PA",
    "STLA.PA",
    "STM.PA",
    "TEPRF.PA",
    "TCFP.PA",
    "TTEF.PA",
    "URW.AS",
    "VIE.PA",
    "SGEF.PA",
    "VIV.PA",
    "WLN.PA",
]

FCHI_SUMMARY_LINKS = [".DJI", "EUR=", "/.STOXX50E", ".FCHI", ".AD.FCHI"]

LSEG_PEERS_INSTRUMENTS = [
    "DB1Gn.DE",
    "ENX.PA",
    "SDR.L",
    "AMUN.PA",
    "DWSG.DE",
    "PGHN.S",
    "ABDN.L",
    "EQTAB.ST",
    "EMG.L",
    "HRGV.L",
    "III.L",
    "ASHM.L",
    "STAN.L",
    "ALLFG.AS",
    "NWG.L",
    "BPTB.L",
    "ANTIN.PA",
    "INGA.AS",
    "BARC.L",
    "UBSG.S",
    "LLOY.L",
    "ABNd.AS",
    "CBKG.DE",
    "BAER.S",
    "CSGN.S",
    "ICP.L",
    "TKOO.PA",
    "DBKGn.DE",
    "SJP.L",
    "HSBA.L",
    "ICE.N",
    "SPGI.N",
    "ERST.VI",
    "NDASE.ST",
    "ADYEN.AS",
    "KBC.BR",
    "NDAQ.OQ",
    "PUBP.PA",
    "COIN.OQ",
    "CME.OQ",
    "DANSKE.CO",
    "CABK.MC",
    "CBOE.Z",
    "VMUK.L",
    "JUP.L",
    "RAT.L",
    "QLT.L",
    "ITX.MC",
    "BBVA.MC",
    "ISP.MI",
]


def check_chain_discovery_object_for_attributes(chain, name, expected_constituents):
    assert chain.name == name, f"Discovery chain contain {chain.name}"
    compare_list(
        chain.constituents, expected_constituents
    ), f"Inconsistency in constituents list"
    assert hasattr(chain, "summary_links")


def is_iterable(obj):
    try:
        iter(obj)
    except TypeError:
        return False
    return True



from typing import TYPE_CHECKING, Callable, List, Union

import pandas as pd

from ..._tools._dataframe import (
    convert_df_columns_to_datetime,
    convert_dtypes,
    convert_str_to_datetime,
)
from ...eikon._tools import tz_replacer

if TYPE_CHECKING:
    from ._data_classes import HeadlineRDP, HeadlineUDF


def news_build_df_udf(content_data, **kwargs) -> pd.DataFrame:
    columns = ["text", "storyId", "sourceCode"]

    raw_headlines = content_data.get("headlines", [])
    index = [
        convert_str_to_datetime(tz_replacer(raw_headline["versionCreated"]))
        for raw_headline in raw_headlines
    ]

    data = [
        [raw_headline[column] for column in columns] for raw_headline in raw_headlines
    ]
    if data:
        df = pd.DataFrame(
            data=data,
            index=index,
            columns=columns,
        )
        df = convert_dtypes(df)

    else:
        df = pd.DataFrame([], index, columns)

    df.index.name = "versionCreated"
    df.rename(columns={"text": "headline"}, inplace=True)
    return df


def news_build_df_rdp(raw: dict, **kwargs) -> pd.DataFrame:
    columns = ["headline", "storyId", "sourceCode"]
    if isinstance(raw, list):
        content_data = []

        for i in raw:
            content_data.extend(i["data"])

    else:
        content_data = raw["data"]

    index = [
        convert_str_to_datetime(
            tz_replacer(headline["newsItem"]["itemMeta"]["versionCreated"]["$"])
        )
        for headline in content_data
    ]

    data = []

    for headline_data in content_data:
        news_item = headline_data.get("newsItem", dict())
        item_meta = news_item.get("itemMeta", {})
        info_sources = news_item["contentMeta"]["infoSource"]
        info_source = next(
            (
                item["_qcode"]
                for item in info_sources
                if item["_role"] == "sRole:source"
            ),
            None,
        )
        data.append(
            [
                item_meta["title"][0]["$"],
                headline_data["storyId"],
                info_source,
            ]
        )
    if data:
        df = pd.DataFrame(
            data=data,
            index=index,
            columns=columns,
        )
        df = convert_dtypes(df)

    else:
        df = pd.DataFrame([], columns=columns)
    df.index.name = "versionCreated"
    return df


def _get_text_from_story(story: dict) -> str:
    news_item = story.get("newsItem", dict())
    content_set = news_item.get("contentSet", dict())
    inline_data = content_set.get("inlineData", [dict()])
    return inline_data[0].get("$")


def _get_headline_from_story(story: dict) -> str:
    news_item = story.get("newsItem", dict())
    content_meta = news_item.get("contentMeta", dict())
    headline = content_meta.get("headline", [dict()])
    return headline[0].get("$")


def get_headlines(
    raw: dict,
    build_headline: Callable[[dict], Union["HeadlineRDP", "HeadlineUDF"]],
    limit: int,
) -> List[Union["HeadlineRDP", "HeadlineUDF"]]:
    headlines = []

    if isinstance(raw, list):

        data = []
        for i in raw:
            data.extend(i.get("data", i.get("headlines", [])))

    else:
        data = raw.get("data", raw.get("headlines", []))

    for datum in data:
        headline = build_headline(datum)
        headlines.append(headline)

    headlines = headlines[:limit]
    return headlines



from inspect import signature
import refinitiv.data.content.ipa.financial_contracts as rdf
from refinitiv.data.delivery._data._data_provider_factory import make_provider
from tests.unit.conftest import StubSession
import pytest

from tests.unit.conftest import (
    remove_dunder_methods,
    has_property_names_in_class,
    get_property_names,
    remove_private_attributes,
)


def test_ipa_cds_definition_attributes():
    expected_attributes = ["get_data", "get_data_async", "get_stream"]
    testing_attributes = dir(rdf.cds.Definition)
    testing_attributes = remove_dunder_methods(testing_attributes)
    testing_attributes = remove_private_attributes(testing_attributes)
    assert expected_attributes == testing_attributes


def test_ipa_cds_attributes():
    expected_attributes = [
        "BusinessDayConvention",
        "CdsConvention",
        "DayCountBasis",
        "Definition",
        "Direction",
        "DocClause",
        "Frequency",
        "PremiumLegDefinition",
        "PricingParameters",
        "ProtectionLegDefinition",
        "Seniority",
        "StubRule",
    ]
    testing_attributes = dir(rdf.cds)
    testing_attributes = remove_dunder_methods(testing_attributes)
    testing_attributes = remove_private_attributes(testing_attributes)
    assert expected_attributes == testing_attributes, testing_attributes


@pytest.mark.parametrize(
    argnames="input_data",
    ids=[
        "Definition",
        "PremiumLegDefinition",
        "ProtectionLegDefinition",
        "PricingParameters",
    ],
    argvalues=[
        (
            rdf.cds._cds_definition.CdsInstrumentDefinition,
            {
                "instrument_tag": "instrument_tag",
                "instrument_code": "instrument_code",
                "cds_convention": rdf.cds.CdsConvention.ISDA,
                "trade_date": "trade_date",
                "step_in_date": "step_in_date",
                "start_date": "start_date",
                "end_date": "end_date",
                "tenor": "tenor",
                "start_date_moving_convention": rdf.cds.BusinessDayConvention.MODIFIED_FOLLOWING,
                "end_date_moving_convention": rdf.cds.BusinessDayConvention.NO_MOVING,
                "adjust_to_isda_end_date": True,
                "protection_leg": rdf.cds.ProtectionLegDefinition(
                    index_factor=1,
                    index_series=1,
                    notional_amount=1,
                    recovery_rate=1,
                    seniority=rdf.cds.Seniority.PREFERENCE,
                    settlement_convention="settlement_convention",
                ),
                "premium_leg": rdf.cds.PremiumLegDefinition(
                    direction=rdf.cds.Direction.PAID,
                    interest_payment_ccy="interest_payment_ccy",
                    interest_payment_frequency=rdf.cds.Frequency.SEMI_ANNUAL,
                    interest_calculation_method=rdf.cds.DayCountBasis.DCB_ACTUAL_360,
                ),
                "accrued_begin_date": "accrued_begin_date",
            },
        ),
        (
            rdf.cds.PremiumLegDefinition,
            {
                "direction": rdf.cds.Direction.PAID,
                "notional_ccy": "notional_ccy",
                "notional_amount": 3.3,
                "fixed_rate_percent": 4.4,
                "interest_payment_frequency": rdf.cds.Frequency.ANNUAL,
                "interest_calculation_method": rdf.cds.DayCountBasis.DCB_30_360,
                "accrued_calculation_method": rdf.cds.DayCountBasis.DCB_30_ACTUAL,
                "payment_business_day_convention": rdf.cds.BusinessDayConvention.NO_MOVING,
                "first_regular_payment_date": "first_regular_payment_date",
                "last_regular_payment_date": "last_regular_payment_date",
                "payment_business_days": "payment_business_days",
                "stub_rule": rdf.cds.StubRule.MATURITY,
                "accrued_paid_on_default": True,
                "interest_payment_ccy": "interest_payment_ccy",
            },
        ),
        (
            rdf.cds.ProtectionLegDefinition,
            {
                "direction": rdf.cds.Direction.RECEIVED,
                "notional_ccy": "notional_ccy",
                "notional_amount": 3.3,
                "doc_clause": rdf.cds.DocClause.EX_RESTRUCT14,
                "seniority": rdf.cds.Seniority.SENIOR_UNSECURED,
                "index_factor": 6.6,
                "index_series": 7,
                "recovery_rate": 8.8,
                "recovery_rate_percent": 9.9,
                "reference_entity": "reference_entity",
                "settlement_convention": "settlement_convention",
            },
        ),
        (
            rdf.cds.PricingParameters,
            {
                "market_data_date": "market_data_date",
                "cash_amount_in_deal_ccy": 1.1,
                "clean_price_percent": 2.2,
                "conventional_spread_bp": 3.3,
                "upfront_amount_in_deal_ccy": 4.4,
                "upfront_percent": 5.5,
                "valuation_date": "valuation_date",
                "report_ccy": "report_ccy",
            },
        ),
    ],
)
def test_parameter(input_data):
    cls, kwargs = input_data
    args_names = list(kwargs.keys())
    inst = cls(**kwargs)

    s = signature(cls.__init__)
    # +1 for (self)
    assert len(s.parameters) == (
        len(args_names) + 1
    ), f"csl={cls}{str(set(get_property_names(cls)) - set(args_names))}"

    assert has_property_names_in_class(cls, args_names), set(args_names) - set(
        get_property_names(cls)
    )

    for k, v in kwargs.items():
        attr = getattr(inst, k)
        assert attr == v, k


def test_request_body():
    # given
    expected_body = {
        "fields": ["1", "2"],
        "universe": [
            {
                "instrumentDefinition": {
                    "endDate": "2032-02-28",
                },
                "instrumentType": "Cds",
                "pricingParameters": {"marketDataDate": "market_data_date"},
                "extended_params": "extended_params",
            }
        ],
    }
    definition = rdf.cds.Definition(
        end_date="2032-02-28",
        fields=["1", "2"],
        pricing_parameters=rdf.cds.PricingParameters(
            market_data_date="market_data_date"
        ),
        extended_params={"extended_params": "extended_params"},
    )

    # when
    content_type = definition._kwargs["__content_type__"]
    provider = make_provider(content_type)
    request = provider.request.create(StubSession(), "url", **definition._kwargs)

    # then
    testing_body = request.json
    assert testing_body == expected_body, expected_body



from typing import List

import pandas as pd
from dateutil import parser
from pandas.tseries.holiday import nearest_workday, Holiday as PandasHoliday

from .._content_data_validator import ContentDataValidator
from .._request_factory import DatesAndCalendarsResponseFactory
from ...._content_data import Data
from ...._content_data_provider import ContentDataProvider
from ....._tools import create_repr, add_periods_datetime_adapter
from ....._types import OptDateTime
from .....content.ipa._content_provider import DatesAndCalendarsRequestFactory
from .....delivery._data._data_provider import ValidatorContainer


class HolidayName:
    def __init__(self, name: str, calendars: list, countries: list):
        self._name = name
        self._calendars = calendars
        self._countries = countries

    @property
    def name(self):
        return self._name

    @property
    def countries(self):
        return self._countries

    @property
    def calendars(self):
        return self._calendars


class Holiday(PandasHoliday):
    _holiday_names = None

    def __init__(
        self,
        date: "OptDateTime" = None,
        name: str = None,
        holiday: dict = None,
        tag: str = "",
    ):
        self._holiday = holiday or {}

        if self._holiday.get("names"):
            name = self._holiday.get("names")[0]["name"]
        elif not name:
            name = "Name not requested"

        if date is not None:
            date = add_periods_datetime_adapter.get_str(date)
        else:
            date = self._holiday.get("date")

        year, month, day = pd.NA, pd.NA, pd.NA
        if date:
            _date = parser.parse(date)
            year, month, day = _date.year, _date.month, _date.day

        PandasHoliday.__init__(
            self,
            name=name,
            year=year,
            month=month,
            day=day,
            observance=nearest_workday,
        )

        self._date = date or self._holiday.get("date", "Date not requested")
        self._tag = tag
        self._countries = self._holiday.get("countries", [])
        self._calendars = self._holiday.get("calendars", [])

    @property
    def date(self):
        return self._date

    @property
    def countries(self):
        return self._countries

    @property
    def calendars(self):
        return self._calendars

    @property
    def names(self) -> List[HolidayName]:
        if self._holiday_names is None:
            self._holiday_names = [
                HolidayName(
                    name=holiday_name["name"],
                    calendars=holiday_name["calendars"],
                    countries=holiday_name["countries"],
                )
                for holiday_name in self._holiday.get("names", [])
            ]
        return self._holiday_names

    @property
    def tag(self):
        return self._tag

    def __repr__(self):
        return create_repr(
            self,
            class_name="HolidayData",
            content="representation of 'holidayOutputs' response",
        )


class HolidaysData(Data):
    _holidays = None

    @property
    def holidays(self) -> List[Holiday]:
        if self._holidays is None:
            self._holidays = [
                Holiday(holiday=holiday, tag=raw_item.get("tag"))
                for raw_item in self.raw
                if not raw_item.get("error")
                for holiday in raw_item["holidays"]
            ]

        return self._holidays


holidays_data_provider = ContentDataProvider(
    request=DatesAndCalendarsRequestFactory(),
    response=DatesAndCalendarsResponseFactory(data_class=HolidaysData),
    validator=ValidatorContainer(content_validator=ContentDataValidator()),
)



def check_story_response_format(response, format):
    html_tags = [
        "html lang",
        "xml:lang",
        "<head>",
        "<body>",
        "<div class=",
        "<style type=",
    ]
    if format.value == "Html":
        assert any(tag in response for tag in html_tags)
    elif format.value == "Text":
        assert not all(tag in response for tag in html_tags)



from enum import Enum, unique

import pytest

from refinitiv.data.content.symbol_conversion import SymbolTypes
from refinitiv.data.content.symbol_conversion._symbol_type import (
    symbol_types_arg_parser,
)


@unique
class SomeEnum(Enum):
    Foo = "foo"
    Buzz = "buzz"


def test_throw_error_when_convert_some_enum():
    # given
    some_enum = SomeEnum.Foo

    # then
    with pytest.raises(AttributeError):
        SymbolTypes.convert_to_str(some_enum)


def test_throw_error_when_convert_incorrect_symbol_str():
    # given
    some_str = "foo"

    # then
    with pytest.raises(AttributeError):
        SymbolTypes.convert_to_str(some_str)


def test_convert_symbol_type_to_correct_str():
    # given
    for sym_type in SymbolTypes:
        # when
        sym_str = symbol_types_arg_parser.get_str(sym_type)

        # then
        assert sym_str == sym_type.value


def test_convert_correct_symbol_str_to_correct_str():
    # given
    for sym_str, item in SymbolTypes.__members__.items():
        # when
        conv_sym_str = symbol_types_arg_parser.get_str(sym_str)

        # then
        assert conv_sym_str == item.value


def test_convert_lower_symbol_str_to_correct_str():
    # given
    for sym_str, item in SymbolTypes.__members__.items():
        sym_str_lower = sym_str.lower()

        # when
        conv_sym_str = symbol_types_arg_parser.get_str(sym_str_lower)

        # then
        assert conv_sym_str == item.value


def test_convert_upper_symbol_str_to_correct_str():
    # given
    for sym_str, item in SymbolTypes.__members__.items():
        sym_str_upper = sym_str.upper()

        # when
        conv_sym_str = symbol_types_arg_parser.get_str(sym_str_upper)

        # then
        assert conv_sym_str == item.value


def test_empty_str_when_normalize_incorrect_symbol_str():
    # given
    sym_str = "RICfooo"

    # then
    with pytest.raises(AttributeError):
        # when
        symbol_types_arg_parser.get_str(sym_str)



from refinitiv.data.delivery._stream import StreamState, StreamEvent
from refinitiv.data.delivery._stream._stream_factory import create_offstream_contrib
from refinitiv.data.delivery._stream.contrib._stream_connection import (
    OffStreamContribConnection,
)
from tests.unit.conftest import StubSession


def test_connection():
    session = StubSession()
    cxn = OffStreamContribConnection(1, "name", session, {})

    assert cxn.can_reconnect is False


def test_stream_contributed_is_set_false():
    session = StubSession(is_open=True)
    stream = create_offstream_contrib(session, "name", "api", "domain", "service")

    assert stream._contributed.is_set() is False


def test_stream_get_next_post_id():
    session = StubSession(is_open=True)
    stream = create_offstream_contrib(session, "name", "api", "domain", "service")

    assert stream.get_next_post_id() == 5


def test_stream_get_contrib_message():
    expected_contrib_message = {
        "Ack": True,
        "Domain": "domain",
        "ID": 2,
        "Key": {"Name": "name", "Service": "service"},
        "Message": {
            "Domain": "domain",
            "Fields": {"BID": 40.83},
            "ID": 0,
            "Type": "Update",
        },
        "PostID": 5,
        "Type": "Post",
    }
    session = StubSession(is_open=True)
    stream = create_offstream_contrib(session, "name", "api", "domain", "service")

    contrib_message = stream.get_contrib_message({"BID": 40.83}, "Update")

    assert contrib_message == expected_contrib_message


def test_stream_open():
    def mock_initialize_cxn():
        stream._cxn = OffStreamContribConnection(1, "name", session, {})
        stream._event = StreamEvent.get(stream.id)

    session = StubSession(is_open=True)
    stream = create_offstream_contrib(session, "name", "api", "domain", "service")
    stream._initialize_cxn = mock_initialize_cxn

    stream_state = stream.open()

    assert stream_state is StreamState.Opened


def test_stream_close():
    def mock_initialize_cxn():
        stream._cxn = OffStreamContribConnection(1, "name", session, {})
        stream._event = StreamEvent.get(stream.id)

    session = StubSession(is_open=True)
    stream = create_offstream_contrib(session, "name", "api", "domain", "service")
    stream._initialize_cxn = mock_initialize_cxn
    stream._release_cxn = lambda: None
    stream.open()

    stream_state = stream.close()

    assert stream_state is StreamState.Closed



__all__ = (
    "get_data",
    "get_news_headlines",
    "get_news_story",
    "get_symbology",
    "get_timeseries",
    "set_app_key",
    "set_log_level",
    "StreamingPrices",
    "TR_Field",
)

from ._data_grid import TR_Field, get_data
from ._news_request import get_news_headlines, get_news_story
from ._symbology import get_symbology
from ._time_series import get_timeseries
from ._tools import set_app_key, set_log_level
from ._streaming_prices import StreamingPrices



import abc
from typing import TYPE_CHECKING, Type, Generic, TypeVar

from ._data_factory import DataFactoryMixin, ABCDataFactory
from ._endpoint_data import EndpointData, ABCData
from ._response import Response, ABCResponse

if TYPE_CHECKING:
    from ._parsed_data import ParsedData

RT = TypeVar("RT")


class ABCResponseFactory(Generic[RT], abc.ABC):
    @abc.abstractmethod
    def create_response(
        self, is_success: bool, parsed_data: "ParsedData", **kwargs
    ) -> RT:
        pass

    @abc.abstractmethod
    def create_success(self, parsed_data: "ParsedData", **kwargs) -> RT:
        pass

    @abc.abstractmethod
    def create_fail(self, parsed_data: "ParsedData", **kwargs) -> RT:
        pass


class ResponseFactory(DataFactoryMixin, ABCResponseFactory[Response]):
    def __init__(
        self,
        response_class: Type[ABCResponse] = None,
        data_class: Type[ABCData] = None,
    ):
        super().__init__()
        self.response_class = response_class or Response
        self.data_class = data_class or EndpointData

    def create_response(
        self, is_success: bool, parsed_data: "ParsedData", **kwargs
    ) -> Response:
        if is_success:
            return self.create_success(parsed_data, **kwargs)
        else:
            return self.create_fail(parsed_data, **kwargs)

    def create_success(self, parsed_data: "ParsedData", **kwargs) -> Response:
        return self.response_class(True, parsed_data, data_factory=self, **kwargs)

    def create_fail(self, parsed_data: "ParsedData", **kwargs) -> Response:
        return self.response_class(False, parsed_data, data_factory=self, **kwargs)



__all__ = (
    "Axis",
    "Definition",
    "EtiCalculationParams",
    "EtiInputVolatilityType",
    "EtiSurfaceDefinition",
    "Format",
    "MaturityFilter",
    "MoneynessType",
    "MoneynessWeight",
    "PriceSide",
    "StrikeFilter",
    "StrikeFilterRange",
    "SurfaceFilters",
    "SurfaceLayout",
    "TimeStamp",
    "VolatilityModel",
)

from ._definition import Definition
from ._enums import (
    EtiInputVolatilityType,
    VolatilityModel,
    PriceSide,
    TimeStamp,
    MoneynessType,
    Axis,
    Format,
)
from ._models import (
    MoneynessWeight,
    SurfaceFilters,
    MaturityFilter,
    StrikeFilterRange,
    StrikeFilter,
    SurfaceLayout,
)
from ..._surfaces._eti_surface_definition import EtiSurfaceDefinition
from ..._surfaces._eti_surface_parameters import (
    EtiSurfaceParameters as EtiCalculationParams,
)



import allure
import pytest

import refinitiv.data.session.desktop as desktop
import tests.integration.conftest as conf
from refinitiv.data import OpenState
from refinitiv.data.session import EventCode
from refinitiv.data._core.session import set_default
from tests.integration.core.session.conftest import (
    assert_session_open_state,
    on_state,
    on_event,
)
from refinitiv.data.delivery import omm_stream
from tests.integration.helpers import (
    compare_list,
    check_event_code,
    check_event_message
)

def check_port_dummy(*args, **kwargs):
    pass


@allure.suite("Session layer - Desktop session")
@allure.feature("Session layer - Desktop session")
@allure.severity(allure.severity_level.CRITICAL)
class TestDesktopSession:
    @allure.title(
        "Verify that desktop session is not opened and error notification received using incorrect application key"
    )
    @pytest.mark.caseid("C39516003")
    def test_session_is_not_opened_using_invalid_credentials(self):
        event_list = []
        session = desktop.Definition(
            app_key="INVALID_APP_KEY",
        ).get_session()
        session.on_event(
            lambda event_code, event_msg, _: on_event(
                event_code, event_msg, session, event_list
            )
        )
        session.open()

        assert event_list[0][0] == EventCode.SessionAuthenticationFailed
        assert event_list[0][1] == "Status code 400: App key is incorrect"
        assert session.open_state == OpenState.Closed

    @allure.title(
        "Verify that desktop session is not opened and error notification received if Eikon is not running"
    )
    @pytest.mark.caseid("C39516004")
    def test_session_is_not_opened_when_eikon_is_not_run(self):
        event_list = []
        session = desktop.Definition(
            app_key=conf.desktop_app_key,
        ).get_session()
        set_default(session)
        session.on_event(
            lambda event_code, event_msg, _: on_event(
                event_code, event_msg, session, event_list
            )
        )
        session._connection.check_proxy = check_port_dummy
        session.open()

        assert event_list[0][0] == EventCode.SessionAuthenticationFailed
        assert event_list[0][1] == "Eikon is not running"
        assert session.open_state == OpenState.Closed
        session.close()

    @allure.title(
        "Open desktop session with callbacks and verify that desktop session is opened"
    )
    @pytest.mark.parametrize(
        "expected_events",
        [[(OpenState.Pending, "Session opening in progress"),
          (OpenState.Opened, "Session is opened"),
          (OpenState.Closed, "Session is closed")]],
    )
    @pytest.mark.caseid("C39515688")
    def test_desktop_session_with_callbacks(self, expected_events):
        event_list = []
        session = desktop.Definition(
            app_key=conf.desktop_app_key,
        ).get_session()
        set_default(session)
        session.on_event(
            lambda event, message, session: on_event(
                event, message, session, event_list
            )
        )
        session.on_state(
            lambda open_state, message, session: on_state(
                open_state, message, session, event_list
            )
        )
        session.open()
        assert_session_open_state(session)

        session.close()
        compare_list(event_list, expected_events)

    @allure.title("Verify using desktop session with ContextManager")
    @pytest.mark.caseid("C44023031")
    @pytest.mark.smoke
    def test_desktop_session_is_opened_and_is_closed_using_context_manager(self):
        session = desktop.Definition(
            app_key=conf.desktop_app_key,
        ).get_session()
        with session:
            assert session.open_state == OpenState.Opened
        assert session.open_state == OpenState.Closed

    @allure.title("Verify using desktop session with asynchronous ContextManager")
    @pytest.mark.caseid("C44023032")
    @pytest.mark.smoke
    async def test_desktop_session_is_opened_and_is_closed_using_async_context_manager(
        self,
    ):
        session = desktop.Definition(
            app_key=conf.desktop_app_key,
        ).get_session()
        async with session:
            assert session.open_state == OpenState.Opened
        assert session.open_state == OpenState.Closed

    @allure.title(
        "Open desktop session with callbacks and verify that all cnx events are received"
    )
    @pytest.mark.parametrize(
        "expected_events",
        [[(OpenState.Pending, "Session opening in progress"),
          (OpenState.Opened, "Session is opened"),
          (EventCode.StreamConnecting, {'url': 'ws://localhost:9060/api/rdp/streaming/pricing/v1/WebSocket', 'api_cfg': 'apis.streaming.pricing.endpoints.main'}),
          (EventCode.StreamConnected, {'url': 'ws://localhost:9060/api/rdp/streaming/pricing/v1/WebSocket', 'api_cfg': 'apis.streaming.pricing.endpoints.main'}),
          (OpenState.Closed, "Session is closed")]],
    )
    @pytest.mark.caseid("C39515688")
    def test_desktop_session_with_callbacks(self, expected_events):
        event_list = []
        session = desktop.Definition(
            app_key=conf.desktop_app_key,
        ).get_session()
        set_default(session)
        session.on_event(
            lambda event, message, session: on_event(
                event, message, session, event_list
            )
        )
        session.on_state(
            lambda open_state, message, session: on_state(
                open_state, message, session, event_list
            )
        )
        session.open()
        stream = omm_stream.Definition(name="GBP=", fields=["BID", "ASK"]).get_stream(session=session)
        stream.open()
        assert stream.open_state == OpenState.Opened, "Stream is not in opened state"
        stream.close()
        assert stream.open_state == OpenState.Closed, "Stream is not in closed state"

        assert_session_open_state(session)
        session.close()
        check_event_code(event_list, expected_events)
        check_event_message(event_list, expected_events)



import allure
import pytest

import refinitiv.data as rd
from refinitiv.data.delivery import cfs
from tests.integration.constants_list import HttpStatusCode, HttpReason
from tests.integration.delivery.cfs.conftest import (
    check_amount_of_objects_in_response_not_bigger_than,
)
from tests.integration.delivery.cfs.packages.conftest import (
    check_all_packages_names_contain_keyword,
    check_all_packages_types,
    check_bucket_name_in_all_packages,
    check_included_total_result_in_response,
    check_included_entitlement_result_in_response,
    check_packages_in_two_responses_are_not_the_same,
    check_packages_in_second_response_are_on_proper_position_in_first_response,
)
from tests.integration.helpers import (
    check_response_status,
    check_non_empty_response_data,
    get_async_response_from_definition,
)


@allure.suite("Delivery object - CFS packages")
@allure.feature("Delivery object - CFS packages")
@allure.severity(allure.severity_level.CRITICAL)
class TestCfsPackages:
    @allure.title("Get packages with no params and default amount 25 - synchronously")
    @pytest.mark.caseid("36839411")
    @pytest.mark.smoke
    def test_get_packages_synchronously(self, open_platform_session):
        response = cfs.packages.Definition().get_data()
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)
        check_amount_of_objects_in_response_not_bigger_than(25, response)

    @allure.title("Get packages with no params and default amount 25 - synchronously")
    @pytest.mark.caseid("36839412")
    async def test_get_packages_asynchronously(self, open_platform_session_async):
        response = await get_async_response_from_definition(cfs.packages.Definition())

        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)
        check_amount_of_objects_in_response_not_bigger_than(25, response)

    @allure.title("Get packages with closed session")
    @pytest.mark.caseid("36839413")
    def test_get_packages_with_closed_session(self):
        session = rd.session.desktop.Definition(app_key=" ").get_session()
        rd.session.set_default(session)
        packages_definition = cfs.packages.Definition()
        with pytest.raises(
            ValueError, match="Session is not opened. Can't send any request"
        ):
            packages_definition.get_data()

    @allure.title("Get packages with existing package_name param (full/partial match)")
    @pytest.mark.parametrize("package_name", ["cfs", "RDP Bulk CFS Package"])
    @pytest.mark.caseid("36839415")
    def test_get_packages_with_package_name_param(
        self, open_platform_session, package_name
    ):
        response = cfs.packages.Definition(package_name=package_name).get_data()
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)
        check_all_packages_names_contain_keyword(response, package_name)

    @allure.title("Get packages with valid package_type param")
    @pytest.mark.parametrize("package_type", ["core"])
    @pytest.mark.caseid("36839422")
    def test_get_packages_with_package_type_param(
        self, open_platform_session, package_type
    ):
        response = cfs.packages.Definition(package_type=package_type).get_data()
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)
        check_all_packages_types(response, package_type)

    @allure.title("Get packages with existing bucket_name param")
    @pytest.mark.parametrize("bucket_name", ["ESG"])
    @pytest.mark.caseid("36839584")
    def test_get_packages_with_bucket_name_param(
        self, open_platform_session, bucket_name
    ):
        response = cfs.packages.Definition(bucket_name=bucket_name).get_data()
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)
        check_bucket_name_in_all_packages(response, bucket_name)

    @allure.title(
        "Get packages with included_total_result and included_entitlement_result params set to true"
    )
    @pytest.mark.caseid("36839589")
    def test_get_packages_with_included_total_result_and_included_entitlement_result_params(
        self, open_platform_session
    ):
        response = cfs.packages.Definition(
            included_total_result=True,
            included_entitlement_result=True,
        ).get_data()
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)
        check_included_total_result_in_response(response)
        check_included_entitlement_result_in_response(response)

    @allure.title(" Get packages with valid page_size param ")
    @pytest.mark.parametrize("page_size", [5])
    @pytest.mark.caseid("36839601")
    def test_get_packages_with_page_size_param(self, open_platform_session, page_size):
        response = cfs.packages.Definition(page_size=page_size).get_data()
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)
        check_amount_of_objects_in_response_not_bigger_than(page_size, response)

    @allure.title("Get packages with valid skip_token parameter")
    @pytest.mark.caseid("36842315")
    def test_get_packages_with_skip_token_param(self, open_platform_session):
        response1 = cfs.packages.Definition().get_data()
        skip_token = response1.data.raw["skip_token"]
        response2 = cfs.packages.Definition(skip_token=skip_token).get_data()
        check_response_status(response2, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response2)
        check_packages_in_two_responses_are_not_the_same(response1, response2)

    @allure.title("Get packages with valid page param")
    @pytest.mark.parametrize("page_size,page", [(10, 2)])
    @pytest.mark.caseid("36839601")
    def test_get_packages_with_page_param(self, open_platform_session, page_size, page):
        response1 = cfs.packages.Definition(page_size=50).get_data()
        response2 = cfs.packages.Definition(page_size=page_size, page=page).get_data()
        check_packages_in_second_response_are_on_proper_position_in_first_response(
            response1, response2, page_size, page
        )



from typing import TYPE_CHECKING

from ._search_data_provider import FilingsSearchData
from ..._content_type import ContentType
from ...delivery._data._data_provider import DataProviderLayer, BaseResponse
from ...delivery._data._endpoint_data import RequestMethod

if TYPE_CHECKING:
    from ..._types import OptDict, OptStr


class Definition(DataProviderLayer[BaseResponse[FilingsSearchData]]):
    """
    This class provide searching and filtering for Filings documents
    Query string can be built by using data-store endpoint in API playgound

    Parameters
    ----------
    query: str
        Full query string to request the search. This will override other parameters.
    variables: dict
        Variables can be used for this query

    Examples
    --------
    >>> from refinitiv.data.content import filings
    >>> query = '{  FinancialFiling(filter: {AND: [{FilingDocument: {DocumentSummary: {FeedName: {EQ: "Edgar"}}}}, {FilingDocument: {DocumentSummary: {FormType: {EQ: "10-Q"}}}}, {FilingDocument: {DocumentSummary: {FilingDate: {BETWN: {FROM: "2020-01-01T00:00:00Z", TO: "2020-12-31T00:00:00Z"}}}}}]}, sort: {FilingDocument: {DocumentSummary: {FilingDate: DESC}}}, limit: 10) {    _metadata {      totalCount    }    FilingDocument {      Identifiers {        Dcn      }      DocId      FinancialFilingId      DocumentSummary {        DocumentTitle        FeedName        FormType        HighLevelCategory        MidLevelCategory        FilingDate        SecAccessionNumber        SizeInBytes          }  FilesMetaData {        FileName        MimeType      }    }  }}'
    >>> definition = filings.search.Definition(query=query)
    >>> response = definition.get_data()
    >>> # response.data.df for prioritize information
    >>> # response.data.raw to see all data responded from the service
    >>> response.data.files[0].download(path="C:\\Downloads\\download_test")
    >>> # To download all files from search
    >>> response.data.files.download()

    >>> # async download files
    >>> await response.data.files[0].download_async()
    >>> await response.data.files.download_async()
    """

    def __init__(self, query: "OptStr" = None, variables: "OptDict" = None):
        self.query = query
        self.variables = variables

        super().__init__(
            ContentType.FILINGS_SEARCH,
            method=RequestMethod.POST,
            query=self.query,
            variables=self.variables,
        )



import json
import os
import sys
import threading
from collections.abc import Iterable
from types import SimpleNamespace

import pytest

from refinitiv.data import _configure as configure
from refinitiv.data._core.session import set_default
from refinitiv.data._core.session._session import Session
from refinitiv.data._core.session._session_cxn_factory import SessionConnection
from refinitiv.data._core.session._session_cxn_type import SessionCxnType
from refinitiv.data._core.session._session_type import SessionType
from refinitiv.data._core.session.grant_password import GrantPassword
from refinitiv.data._open_state import OpenState
from refinitiv.data.delivery._stream import stream_cxn_cache, get_cxn_cfg_provider

PROJECT_CONFIG_PATH = os.path.join(os.getcwd(), configure._default_config_file_name)


@pytest.fixture(autouse=True, scope="session")
def setup_teardown_session(request):
    print("\nDoing setup")

    def fin():
        print("\nDoing teardown")
        import threading

        s = "\n\t".join([str(t) for t in threading.enumerate()])
        print(f"Threads:\n\t{s}")

    request.addfinalizer(fin)


@pytest.fixture(autouse=True)
def setup_teardown_function():
    import refinitiv.data as rd

    rd.session.set_default(None)

    cxns = stream_cxn_cache.get_all_cxns()
    for c in cxns:
        c.dispose()
        c.join()


@pytest.fixture(scope="function")
def write_project_config(project_config_path):
    def inner(arg):
        if isinstance(arg, dict):
            arg = json.dumps(arg)
        f = open(project_config_path, "w")
        f.write(arg)
        f.close()
        return project_config_path

    yield inner

    configure._dispose()

    remove_project_config()


@pytest.fixture()
def project_config_path():
    return PROJECT_CONFIG_PATH


def remove_project_config():
    remove_config(PROJECT_CONFIG_PATH)


def remove_config(path):
    while os.path.exists(path):
        try:
            os.remove(path)
        except (PermissionError, FileNotFoundError):
            pass


def is_dunder_method(s):
    return s.startswith("__") and s.endswith("__")


def remove_dunder_methods(iterable):
    return [item for item in iterable if not is_dunder_method(item)]


def assert_error(e):
    expected_message = "__init__() got an unexpected keyword argument 'extended_params'"
    if sys.version_info >= (3, 10):
        expected_message = (
            "Definition.__init__() got an unexpected keyword argument 'extended_params'"
        )
    testing_message = str(e)
    assert testing_message == expected_message, e


def kwargs(**_):
    return _


def args(**_):
    return _.values()


def remove_private_attributes(iterable):
    return [item for item in iterable if not item.startswith("_")]


def get_property_names(cls):
    return [p for p in dir(cls) if isinstance(getattr(cls, p, None), property)]


def has_property_names_in_class(cls, expected_property_names):
    if isinstance(expected_property_names, dict):
        expected_property_names = list(expected_property_names.keys())
    return set(get_property_names(cls)) == set(expected_property_names)


def load_json(filepath):
    with open(filepath, "r") as f:
        s = f.read()
        o = json.loads(s)
    return o


class StubConfig(dict):
    def __init__(self, d=None) -> None:
        super().__init__()
        d = d or {}
        for k, v in d.items():
            self[k] = v

    def get_str(self, key):
        return str(self[key])

    def remove_listener(self, *args, **kwargs):
        pass

    def get_list(self, key):
        return list(self[key])

    def get_param(self, key):
        try:
            return self[key]
        except KeyError:
            raise AttributeError(f"Config object doesn't have '{key}' attribute")


class StubConfigure(object):
    def __init__(self, config=None) -> None:
        super().__init__()
        config = config or StubConfig()
        self.config = config

        keys = SimpleNamespace()
        keys.platform_session = lambda key: f"platform_{key}"
        keys.desktop_session = lambda key: f"desktop_{key}"
        keys.log_file_enabled = "key_log_file_enabled"
        keys.log_console_enabled = "key_log_console_enabled"
        keys.log_level = "key_log_level"
        keys.log_filter = "key_log_filter"
        self.keys = keys

    def get(self, key, default=None):
        if "platform" in key and self.config.get("platform_config"):
            return self.config.get("platform_config")

        if "desktop" in key and self.config.get("desktop_config"):
            return self.config.get("desktop_config")

        if self.config.get(key):
            return self.config.get(key)

        return default

    def get_str(self, key):
        return str(self.get(key))


class StubResponse:
    def __init__(self, content_data=None, status_code=None, mocked_json=None) -> None:
        self.status_code = status_code or 200
        self.text = "test_text"

        request = SimpleNamespace()
        request.headers = {}
        request.url = SimpleNamespace()
        request.url.path = "test_url"

        self.request = request

        self.http_headers = {"content-type": "application/json; charset=utf-8"}
        self.headers = self.http_headers

        self._json = mocked_json or {"data": [{}], "headers": []}
        if content_data:
            self._json = content_data

    def json(self):
        return self._json


class StubSession(Session):
    type = SessionType.NONE

    def __init__(
        self,
        is_open=False,
        config=None,
        response=None,
        fail_if_error=False,
        deployed_platform_host=None,
        session_cxn_type=SessionCxnType.REFINITIV_DATA,
        stream_auto_reconnection=False,
    ) -> None:
        super().__init__(app_key="app_key")

        self._session_cxn_type = session_cxn_type
        self._deployed_platform_host = deployed_platform_host

        def error(message):
            if fail_if_error:
                pytest.fail(str(message))
            else:
                print(message)

        if isinstance(response, list):
            response = iter(response)

        self._response = response
        self.log = lambda *_, **__: None
        self.info = lambda *_, **__: None
        self.debug = lambda *_, **__: None
        self.error = error
        self.exception = error
        self.warning = lambda *_, **__: None
        self._log = lambda *_, **__: None
        self._info = lambda *_, **__: None
        self._debug = lambda *_, **__: None
        self._error = error
        self._exception = error
        self._warning = lambda *_, **__: None

        logger = SimpleNamespace()
        logger.handlers = []
        logger.log = lambda *_, **__: None
        logger.info = lambda *_, **__: None
        logger.debug = lambda *_, **__: None
        logger.error = error
        logger.exception = error
        logger.warning = lambda *_, **__: None
        self._logger = logger
        self._grant = GrantPassword(username="username", password="password")
        import random

        self._logger.name = f"StubSession_{random.random()}"

        self._is_open = is_open

        self._config = config
        if config is None:
            from refinitiv.data import _configure

            self._config = _configure.get_config()

        self._async_mode = False
        self.http_responses = success_http_response_generator()

        self._stream_auto_reconnection = stream_auto_reconnection

    @property
    def signon_control(self):
        return True

    @property
    def stream_auto_reconnection(self):
        return self._stream_auto_reconnection

    @property
    def authentication_token_endpoint_url(self):
        return ""

    @property
    def server_mode(self):
        return False

    @property
    def _connection(self) -> "SessionConnection":
        connection = SimpleNamespace()
        return connection

    @property
    def is_open(self):
        return self._is_open

    @property
    def is_closed(self):
        return not self._is_open

    @property
    def config(self):
        return self._config

    def logger(self):
        return self._logger

    @property
    def open_state(self):
        return OpenState.Opened if self.is_open else OpenState.Closed

    def _get_rdp_url_root(self):
        return "test_get_rdp_url_root"

    def _get_udf_url(self):
        return "test_get_udf_url"

    async def http_request_async(self, *_, **__):
        if self.async_mode:
            return next(self.http_responses)
        elif isinstance(self._response, Iterable):
            return next(self._response)
        else:
            return self._response or StubResponse()

    def http_request(self, *_, **__):
        if self.async_mode:
            return next(self.http_responses)
        elif isinstance(self._response, Iterable):
            return next(self._response)
        else:
            return self._response or StubResponse()

    async def open_async(self) -> OpenState:
        # do nothing
        pass

    def close(self):
        self._is_open = False
        stream_cxn_cache.close_cxns(self)

    @property
    def async_mode(self):
        return self._async_mode

    @async_mode.setter
    def async_mode(self, value: bool):
        self._async_mode = bool(value)

    def get_omm_login_message(self):
        return {}

    def _get_session_cxn_type(self) -> SessionCxnType:
        return self._session_cxn_type

    def get_rdp_login_message(self, stream_id):
        return {}


class Stub404OperationResponse(StubResponse):
    def __init__(self) -> None:
        super().__init__()

        self.status_code = 404
        self.reason = "Page not found"
        self.text = "Not found"


class Stub202AcceptedRequestResponse(StubResponse):
    def __init__(self) -> None:
        super().__init__()

        self.status_code = 202
        self.text = "Accepted"
        self.http_headers.update({"location": "location_url"})


class Stub202NoLocationRequestResponse(StubResponse):
    def __init__(self) -> None:
        super().__init__()

        self.status_code = 202
        self.reason = "accepted"
        self.text = "Accepted"


class Stub200NotStartedOperationResponse(StubResponse):
    def __init__(self) -> None:
        super().__init__()

        self.status_code = 200
        self.reason = "not_started"
        self.text = "Not started"
        self._json = {"status": "not_started"}


class Stub200RunningOperationResponse(StubResponse):
    def __init__(self) -> None:
        super().__init__()

        self.status_code = 200
        self.reason = "OK"
        self.text = "Running"
        self._json.update({"status": "running"})


class StubNoPermissionsResponse(StubResponse):
    def __init__(self) -> None:
        super().__init__()

        self.status_code = 200
        self.text = ""
        self._json = [
            {
                "universe": {"ric": "VOD.L"},
                "status": {
                    "code": "TS.Intraday.UserNotPermission.92000",
                    "message": "User has no permission.",
                },
            }
        ]


SUCCESS_RESPONSE_ID = "success_response_id"
FAILED_RESPONSE_ID = "failed_response_id"


class Stub403AccessDeniedResponse(StubResponse):
    def __init__(self) -> None:
        super().__init__()

        self.status_code = 403
        self._json = {"error": {"message": "access denied."}}


class Stub200SucceededOperationResponse(StubResponse):
    def __init__(self) -> None:
        super().__init__()

        self.status_code = 200
        self.reason = "OK"
        self.text = "Succeeded"

        self._json.update(
            {"status": "succeeded", "resourceLocation": SUCCESS_RESPONSE_ID}
        )


class Stub200FailedOperationResponse(StubResponse):
    def __init__(self) -> None:
        super().__init__()

        self.status_code = 200
        self.reason = "OK"
        self.text = "Failed"

        self._json = {"status": "failed", "resourceLocation": FAILED_RESPONSE_ID}


class StubFailedResponse(StubResponse):
    def __init__(self) -> None:
        super().__init__()

        self.status_code = 400
        self.status = "failed"
        self.text = "Failed"

        self._json = {
            "error": {
                "id": FAILED_RESPONSE_ID,
                "code": "UNKNOWN_CODE",
                "status": "Bad Request",
                "message": "{id=03579501-f9dd-4bea-a9a3-bb115c882d8a, status=Error, message=Invalid input: Unsupported 'instrumentType' value., code=QPS-DPS.Error_InvalidInput_UnsupportedInstrumentType}",
            }
        }


def success_http_response_generator():
    for response in [
        Stub202AcceptedRequestResponse(),
        Stub200RunningOperationResponse(),
        Stub200SucceededOperationResponse(),
        StubResponse(),
    ]:
        yield response


def error_404_http_response_generator():
    for response in [Stub404OperationResponse()]:
        yield response


def error_403_access_denied_http_response_generator():
    for response in [Stub403AccessDeniedResponse()]:
        yield response


def error_no_location_http_response_generator():
    for response in [Stub202NoLocationRequestResponse()]:
        yield response


def error_request_failed_http_response_generator():
    for response in [
        Stub202AcceptedRequestResponse(),
        Stub200RunningOperationResponse(),
        Stub200FailedOperationResponse(),
        StubFailedResponse(),
    ]:
        yield response


def error_user_has_no_permissions():
    for response in [StubNoPermissionsResponse()]:
        yield response


def parametrize_with_test_case(args_names, test_cases, *test_case_ids):
    return pytest.mark.parametrize(
        args_names,
        [
            pytest.param(*test_case_data, id=test_case_id)
            for test_case_id, test_case_data in test_cases.items()
            if test_case_id in test_case_ids
        ],
    )


def send_ws_messages(session, stream_data):
    cxn_cfg_provider = get_cxn_cfg_provider(session)
    cxn_cfg_provider.start_connecting()

    cxn, ws = None, None
    stream_cxn_cache.cxn_created.clear()

    cxn_ready = threading.Event()

    def get_cxn_ready():
        nonlocal cxn, ws
        stream_cxn_cache.cxn_created.wait()
        cxn, *_ = stream_cxn_cache.get_cxns(session)
        cxn._listener_created.wait()
        ws = cxn._listener.ws
        ws.cmd("on_open")
        message = '[{"State": {"Stream": "Open"}}]'
        ws.cmd("on_message", message)
        cxn_ready.set()

    def send():
        cxn_ready.wait()
        for message in stream_data:
            ws.cmd("on_message", json.dumps([message]))

    set_default(session)
    if stream_data:
        func_name = sys._getframe().f_code.co_name
        threading.Thread(
            name=f"{func_name}-1", target=get_cxn_ready, daemon=True
        ).start()
        threading.Thread(name=f"{func_name}-2", target=send, daemon=True).start()


class StubLogger:
    def __init__(self) -> None:
        dummy = lambda *args, **kwargs: None
        self.log = dummy
        self.warning = dummy
        self.error = dummy
        self.debug = dummy
        self.info = dummy



# coding: utf8
from typing import TYPE_CHECKING

from .._object_definition import ObjectDefinition
from ._zc_curve_parameters import ZcCurveParameters
from ._models import Constituents
from ._zc_curve_definitions import ZcCurveDefinitions
from ...._types import OptStr

if TYPE_CHECKING:
    from ._zc_curve_types import CurveDefinition, CurveParameters, OptConstituents


class ZcCurveRequestItem(ObjectDefinition):
    """
    Parameters
    ----------
    constituents : Constituents, optional

    curve_definition : ZcCurveDefinitions, optional

    curve_parameters : ZcCurveParameters, optional

    curve_tag : str, optional

    """

    def __init__(
        self,
        constituents: "OptConstituents" = None,
        curve_definition: "CurveDefinition" = None,
        curve_parameters: "CurveParameters" = None,
        curve_tag: OptStr = None,
    ) -> None:
        super().__init__()
        self.constituents = constituents
        self.curve_definition = curve_definition
        self.curve_parameters = curve_parameters
        self.curve_tag = curve_tag

    @property
    def constituents(self):
        """
        :return: object Constituents
        """
        return self._get_object_parameter(Constituents, "constituents")

    @constituents.setter
    def constituents(self, value):
        self._set_object_parameter(Constituents, "constituents", value)

    @property
    def curve_definition(self):
        """
        :return: object ZcCurveDefinitions
        """
        return self._get_object_parameter(ZcCurveDefinitions, "curveDefinition")

    @curve_definition.setter
    def curve_definition(self, value):
        self._set_object_parameter(ZcCurveDefinitions, "curveDefinition", value)

    @property
    def curve_parameters(self):
        """
        :return: object ZcCurveParameters
        """
        return self._get_object_parameter(ZcCurveParameters, "curveParameters")

    @curve_parameters.setter
    def curve_parameters(self, value):
        self._set_object_parameter(ZcCurveParameters, "curveParameters", value)

    @property
    def curve_tag(self):
        """
        :return: str
        """
        return self._get_parameter("curveTag")

    @curve_tag.setter
    def curve_tag(self, value):
        self._set_parameter("curveTag", value)



from typing import Any, Callable as _Callable, Optional, TYPE_CHECKING

from ._stream import TradeDataStream
from ..._core.session import get_valid_session
from ..._tools import cached_property, create_repr, make_callback
from ...delivery._stream.base_stream import StreamOpenMixin

if TYPE_CHECKING:
    from ..._types import OptStrStrs, ExtendedParams, OptStr
    from ..._core.session import Session
    from ._stream import FinalizedOrders, UniverseTypes, Events


class Stream(StreamOpenMixin):
    def __init__(
        self,
        session: Optional["Session"] = None,
        universe: "OptStrStrs" = None,
        universe_type: Optional["UniverseTypes"] = None,
        fields: "OptStrStrs" = None,
        events: Optional["Events"] = None,
        finalized_orders: Optional["FinalizedOrders"] = None,
        filters: "OptStrStrs" = None,
        api: "OptStr" = None,
        extended_params: "ExtendedParams" = None,
    ):
        self._session = get_valid_session(session)
        self._always_use_default_session = session is None
        self._universe = universe
        self._fields = fields
        self._extended_params = extended_params
        self._universe_type = universe_type
        self._events = events
        self._finalized_orders = finalized_orders
        self._filters = filters
        self._api = api

    @cached_property
    def _stream(self):
        return TradeDataStream(
            session=self._session,
            universe=self._universe,
            fields=self._fields,
            extended_params=self._extended_params,
            universe_type=self._universe_type,
            events=self._events,
            finalized_orders=self._finalized_orders,
            filters=self._filters,
            api=self._api,
        )

    def on_update(self, on_update: _Callable[[dict, "Stream"], Any]) -> "Stream":
        """
        These notifications are emitted when fields of the requested instrument change

        Parameters
        ----------
        on_update : Callable
            Callable object to process retrieved data

        Returns
        -------
        current instance

        Examples
        -------
        >>> import datetime
        >>> from refinitiv.data.content import trade_data_service
        >>>
        >>> def display_event(event, stream):
        ...    current_time = datetime.datetime.now().time()
        ...    print("----------------------------------------------------------")
        ...    print(">>> {} event received at {}".format("Update", current_time))
        ...    print(event)
        >>>
        >>> definition = trade_data_service.Definition(
        ...     universe=[],
        ...     fields=[
        ...        "OrderKey",
        ...        "OrderTime",
        ...        "RIC",
        ...        "Side",
        ...        "AveragePrice",
        ...        "OrderStatus",
        ...        "OrderQuantity",
        ...],
        ...     events=trade_data_service.Events.Full,
        ...     finalized_orders=trade_data_service.FinalizedOrders.P1D
        ...)
        >>> stream = definition.get_stream()
        >>> stream.on_update(display_event)
        >>> stream.open()
        """
        self._stream.on_update = make_callback(on_update)
        return self

    def on_complete(self, on_complete: _Callable[["Stream"], Any]) -> "Stream":
        """
        Full data of requested universe items

        Parameters
        ----------
        on_complete : Callable
            Callable object to process retrieved data

        Returns
        -------
        current instance

        Examples
        -------
        >>> import datetime
        >>> from refinitiv.data.content import trade_data_service
        >>>
        >>> def display_event(stream):
        ...    current_time = datetime.datetime.now().time()
        ...    print("----------------------------------------------------------")
        ...    print(">>> {} event received at {}".format("Complete", current_time))
        >>>
        >>> definition = trade_data_service.Definition(
        ...     universe=[],
        ...     fields=[
        ...        "OrderKey",
        ...        "OrderTime",
        ...        "RIC",
        ...        "Side",
        ...        "AveragePrice",
        ...        "OrderStatus",
        ...        "OrderQuantity",
        ...],
        ...     events=trade_data_service.Events.Full,
        ...     finalized_orders=trade_data_service.FinalizedOrders.P1D
        ...)
        >>> stream = definition.get_stream()
        >>> stream.on_complete(display_event)
        >>> stream.open()
        """
        self._stream.on_complete = on_complete
        return self

    def on_add(self, on_add: _Callable[[dict, "Stream"], Any]) -> "Stream":
        """
        These notifications are sent when the status of one of the requested instruments
        is added

        Parameters
        ----------
        on_add : Callable
            Callable object to process retrieved data

        Returns
        -------
        current instance

        Examples
        -------
        >>> import datetime
        >>> from refinitiv.data.content import trade_data_service
        >>>
        >>> def display_event(event, stream):
        ...    current_time = datetime.datetime.now().time()
        ...    print("----------------------------------------------------------")
        ...    print(">>> {} event received at {}".format("Add", current_time))
        ...    print(event)
        >>>
        >>> definition = trade_data_service.Definition(
        ...     universe=[],
        ...     fields=[
        ...        "OrderKey",
        ...        "OrderTime",
        ...        "RIC",
        ...        "Side",
        ...        "AveragePrice",
        ...        "OrderStatus",
        ...        "OrderQuantity",
        ...],
        ...     events=trade_data_service.Events.Full,
        ...     finalized_orders=trade_data_service.FinalizedOrders.P1D
        ...)
        >>> stream = definition.get_stream()
        >>> stream.on_add(display_event)
        >>> stream.open()
        """
        self._stream.on_add = make_callback(on_add)
        return self

    def on_remove(self, on_remove: _Callable[[dict, "Stream"], Any]) -> "Stream":
        """
        Called when the stream on summary order of universe is removed by the server.
        This callback is called with the reference to the stream object and
        the universe removed.

        Parameters
        ----------
        on_remove : Callable
            Callable object to process retrieved data

        Returns
        -------
        current instance

        Examples
        -------
        >>> import datetime
        >>> from refinitiv.data.content import trade_data_service
        >>>
        >>> def display_event(event, stream):
        ...    current_time = datetime.datetime.now().time()
        ...    print("----------------------------------------------------------")
        ...    print(">>> {} event received at {}".format("Remove", current_time))
        ...    print(event)
        >>>
        >>> definition = trade_data_service.Definition(
        ...     universe=[],
        ...     fields=[
        ...        "OrderKey",
        ...        "OrderTime",
        ...        "RIC",
        ...        "Side",
        ...        "AveragePrice",
        ...        "OrderStatus",
        ...        "OrderQuantity",
        ...],
        ...     events=trade_data_service.Events.Full,
        ...     finalized_orders=trade_data_service.FinalizedOrders.P1D
        ...)
        >>> stream = definition.get_stream()
        >>> stream.on_remove(display_event)
        >>> stream.open()
        """
        self._stream.on_remove = make_callback(on_remove)
        return self

    def on_event(self, on_event: _Callable[[dict, "Stream"], Any]) -> "Stream":
        """
        These notifications are emitted when the status
        of one of the requested instruments changes

        Parameters
        ----------
        on_event

        Returns
        -------
        current instance

        Examples
        -------
        >>> import datetime
        >>> from refinitiv.data.content import trade_data_service
        >>>
        >>> def display_event(event, stream):
        ...    current_time = datetime.datetime.now().time()
        ...    print("----------------------------------------------------------")
        ...    print(">>> {} event received at {}".format("Event", current_time))
        ...    print(event)
        >>>
        >>> definition = trade_data_service.Definition(
        ...     universe=[],
        ...     fields=[
        ...        "OrderKey",
        ...        "OrderTime",
        ...        "RIC",
        ...        "Side",
        ...        "AveragePrice",
        ...        "OrderStatus",
        ...        "OrderQuantity",
        ...],
        ...     events=trade_data_service.Events.Full,
        ...     finalized_orders=trade_data_service.FinalizedOrders.P1D
        ...)
        >>> stream = definition.get_stream()
        >>> stream.on_event(display_event)
        >>> stream.open()
        """
        self._stream.on_event = make_callback(on_event)
        return self

    def on_state(self, on_state: _Callable[[dict, "Stream"], Any]) -> "Stream":
        """
        These notifications are emitted when the state of one of the requested changes

        Parameters
        ----------
        on_state : Callable

        Returns
        -------
        current instance

        Examples
        -------
        >>> import datetime
        >>> from refinitiv.data.content import trade_data_service
        >>>
        >>> def display_event(event, stream):
        ...    current_time = datetime.datetime.now().time()
        ...    print("----------------------------------------------------------")
        ...    print(">>> {} event received at {}".format("State", current_time))
        ...    print(event)
        >>>
        >>> definition = trade_data_service.Definition(
        ...     universe=[],
        ...     fields=[
        ...        "OrderKey",
        ...        "OrderTime",
        ...        "RIC",
        ...        "Side",
        ...        "AveragePrice",
        ...        "OrderStatus",
        ...        "OrderQuantity",
        ...],
        ...     events=trade_data_service.Events.Full,
        ...     finalized_orders=trade_data_service.FinalizedOrders.P1D
        ...)
        >>> stream = definition.get_stream()
        >>> stream.on_state(display_event)
        >>> stream.open()
        """
        self._stream.on_state = make_callback(on_state)
        return self

    def __repr__(self):
        return create_repr(self, class_name="Stream")



from typing import TYPE_CHECKING

from ...._object_definition import ObjectDefinition

if TYPE_CHECKING:
    from ......_types import OptStr


class FxSpotInstrumentDefinition(ObjectDefinition):
    """
    Creates the Cross Currency curve definition with the definition provided.

    Parameters
    ----------
    instrument_code : str, optional
        The code used to define the instrument.
    synthetic_instrument_code : str, optional
        The code used to define the formula.
    template : str, optional
        A reference to a style used to define the instrument.
    """

    def __init__(
        self,
        instrument_code: "OptStr" = None,
        synthetic_instrument_code: "OptStr" = None,
        template: "OptStr" = None,
    ) -> None:
        super().__init__()
        self.instrument_code = instrument_code
        self.synthetic_instrument_code = synthetic_instrument_code
        self.template = template

    @property
    def instrument_code(self):
        """
        The code used to define the instrument.
        :return: str
        """
        return self._get_parameter("instrumentCode")

    @instrument_code.setter
    def instrument_code(self, value):
        self._set_parameter("instrumentCode", value)

    @property
    def synthetic_instrument_code(self):
        """
        The code used to define the formula.
        :return: str
        """
        return self._get_parameter("syntheticInstrumentCode")

    @synthetic_instrument_code.setter
    def synthetic_instrument_code(self, value):
        self._set_parameter("syntheticInstrumentCode", value)

    @property
    def template(self):
        """
        A reference to a style used to define the instrument.
        :return: str
        """
        return self._get_parameter("template")

    @template.setter
    def template(self, value):
        self._set_parameter("template", value)



import pytest
from unittest.mock import MagicMock

from refinitiv.data._tools._common import (
    is_all_defined,
    is_any_defined,
    is_int,
    make_callback,
    urljoin,
    validate_types,
    CallbackHandler,
    bool_or_none_to_str,
    get_correct_filename,
)


@pytest.mark.parametrize(
    "input_value, expected",
    [
        ("1", True),
        (1, True),
        ({}, False),
        (None, False),
        ("1asb", False),
        ("", False),
    ],
)
def test_is_int(input_value, expected):
    # given

    # when
    testing_value = is_int(input_value)

    # then
    assert testing_value is expected, testing_value


@pytest.mark.parametrize(
    "input_value, expected",
    [
        (
            ("https://api.refinitiv.com", "data/news/beta1"),
            "https://api.refinitiv.com/data/news/beta1",
        ),
        (
            ("https://api.refinitiv.com/", "data/news/beta1"),
            "https://api.refinitiv.com/data/news/beta1",
        ),
        (
            ("https://api.refinitiv.com/", "/data/news/beta1"),
            "https://api.refinitiv.com/data/news/beta1",
        ),
        (
            ("https://api.refinitiv.com/", "/data/news/beta1/"),
            "https://api.refinitiv.com/data/news/beta1/",
        ),
        (("https://api.refinitiv.com",), "https://api.refinitiv.com"),
        (("/", "data/news/beta1"), "/data/news/beta1"),
        (("/", "/data/news/beta1"), "/data/news/beta1"),
        (("data/news/beta1", "/"), "data/news/beta1/"),
        (("data/news/beta1/", "/"), "data/news/beta1/"),
        (("/data/news/beta1", "view"), "/data/news/beta1/view"),
        (("data/news/beta1/", "/{universe}"), "data/news/beta1/{universe}"),
    ],
)
def test_urljoin(input_value, expected):
    # given

    # when
    testing_url = urljoin(*input_value)

    # then
    assert testing_url == expected


def test_urljoin_without_arguments_return_empty_string():
    # given
    expected = ""

    # when
    testing_url = urljoin()

    # then
    assert testing_url == expected, testing_url


@pytest.mark.parametrize(
    "input_value, expected",
    [
        (([1], [1]), True),
        (({1}, {1}), True),
        (({1: 1}, {1: 1}), True),
        (("1", "1"), True),
        ((1, 1), True),
        ((True, True), True),
        (([], [1]), False),
        (
            (
                set(),
                {
                    1,
                },
            ),
            False,
        ),
        (({}, {1: 1}), False),
        (("", "1"), False),
        ((0, 1), False),
        ((False, True), False),
    ],
)
def test_is_all_defined(input_value, expected):
    # given

    # when
    testing_url = is_all_defined(*input_value)

    # then
    assert testing_url == expected, testing_url


@pytest.mark.parametrize(
    "input_value, expected",
    [
        (([1], set(), {}, "", 0, False), True),
        (([], {1}, {}, "", 0, False), True),
        (([], set(), {1: 1}, "", 0, False), True),
        (([], set(), {}, "1", 0, False), True),
        (([], set(), {}, "", 1, False), True),
        (([], set(), {}, "", 0, True), True),
        (([], set(), {}, "", 0, False), False),
    ],
)
def test_is_any_defined(input_value, expected):
    # given

    # when
    testing_url = is_any_defined(*input_value)

    # then
    assert testing_url == expected, testing_url


@pytest.mark.parametrize(
    "value, types",
    [[1, [int, str]], [None, [int, str, type(None)]], [True, [bool, str]]],
)
def test_validate_types(value, types):
    validate_types(value, types)


@pytest.mark.parametrize(
    "value, types, message",
    [
        [
            1,
            [bool, str],
            "Parameter '' of invalid type provided: 'int', "
            "expected types: ['bool', 'str']",
        ],
        [
            None,
            [int, str],
            "Parameter '' of invalid type provided: 'NoneType', "
            "expected types: ['int', 'str']",
        ],
        [
            True,
            [int, str],
            "Parameter '' of invalid type provided: 'bool', "
            "expected types: ['int', 'str']",
        ],
        [
            None,
            [int, None],
            "Use 'type(None)' instead 'None', in 'types'",
        ],
    ],
)
def test_validate_types_exception(value, types, message):
    with pytest.raises(TypeError) as exc:
        validate_types(value, types)
    assert str(exc.value) == message


def test_make_callback():
    stream = MagicMock(name="stream")
    arg = {}
    func = MagicMock(name="func")
    my_callback = make_callback(func)
    my_callback(stream, arg)
    func.assert_called_once_with(arg, stream)


def test_callback_called():
    func = MagicMock(name="func")
    test_args = ("test", 1, [])
    handler = CallbackHandler()

    handler.on("test", func)
    handler.emit("test", *test_args)

    func.assert_called_once_with(*test_args)


def test_callback_not_called():
    func = MagicMock(name="func")
    test_args = ("test", 1, [])
    handler = CallbackHandler()

    handler.on("test", func)
    handler.remove_listener("test", func)
    handler.emit("test", *test_args)

    func.assert_not_called()


def test_callback_replaced():
    func = MagicMock(name="func")
    func2 = MagicMock(name="func2")
    test_args = ("test", 1, [])
    handler = CallbackHandler()

    handler.on("test", func)
    handler.on("test", func2)
    handler.emit("test", *test_args)

    func.assert_not_called()
    func2.assert_called_once_with(*test_args)


def test_all_callbacks_called():
    func = [MagicMock(name=f"func{i}") for i in range(10)]
    test_args = ("test", 1, [])
    handler = CallbackHandler(max_listeners=10)

    for f in func:
        handler.on("test", f)
    handler.emit("test", *test_args)

    for f in func:
        f.assert_called_once_with(*test_args)


def test_callback_called_multiple():
    func = MagicMock(name="func")
    test_args = ("test", 1, [])
    handler = CallbackHandler()
    test_call_count = 10

    handler.on("test", func)
    for _ in range(test_call_count):
        handler.emit("test", *test_args)

    assert func.call_count == test_call_count


def test_all_callbacks_removed():
    func = [MagicMock(name=f"func{i}") for i in range(10)]
    test_args = ("test", 1, [])
    handler = CallbackHandler(max_listeners=10)

    for f in func:
        handler.on("test", f)
    handler.remove_all_listeners()
    handler.emit("test", *test_args)

    for f in func:
        f.assert_not_called()


def test_all_callbacks_by_event_removed():
    func1 = MagicMock(name="func1")
    funcs = [MagicMock(name=f"func{i}") for i in range(10)]
    test_args = ("test", 1, [])
    handler = CallbackHandler(max_listeners=10)

    for f in funcs:
        handler.on("test", f)
    handler.on("test2", func1)
    handler.remove_all_listeners("test")
    handler.emit("test", *test_args)
    handler.emit("test2", *test_args)

    for f in funcs:
        f.assert_not_called()
    func1.assert_called_once_with(*test_args)


@pytest.mark.parametrize(
    "input_value, expected",
    [
        [True, "true"],
        [False, "false"],
        [1, 1],
        [0, 0],
        [None, ""],
        ["test", "test"],
        [[], []],
    ],
)
def test_bool_or_none_to_str(input_value, expected):
    testing_result = bool_or_none_to_str(input_value)

    assert testing_result == expected


@pytest.mark.parametrize(
    ("test_value", "expected_result"),
    [
        (
            "RFT-DSP-VotingRights-v2-Delta15-2021-08-16T13:30:36.571Z.jsonl.gz",
            "RFT-DSP-VotingRights-v2-Delta15-2021-08-16T133036.571Z.jsonl.gz",
        ),
        (
            'RFT-DSP-VotingRights-v2-Delta15-2021-08-16T13:30:36."571Z.jsonl.gz',
            "RFT-DSP-VotingRights-v2-Delta15-2021-08-16T133036.571Z.jsonl.gz",
        ),
        (
            'RFT-DSP-VotingRights|v2-Delta15-2021-08-16T13:30:36."571Z.jsonl.gz',
            "RFT-DSP-VotingRightsv2-Delta15-2021-08-16T133036.571Z.jsonl.gz",
        ),
        ("file?name.json", "filename.json"),
    ],
)
def test_get_correct_filename(test_value, expected_result):
    # given
    # when
    result = get_correct_filename(test_value)

    # then
    assert result == expected_result



import pytest

from refinitiv.data._core.session._session_cxn_type import SessionCxnType
from refinitiv.data.delivery._stream._stream_cxn_config_provider import (
    PlatformAndDeployedCxnConfigProvider,
)
from tests.unit.conftest import StubSession, StubConfig, StubResponse, args


def create_platform_and_deployed_cxn_config_provider(rawconfig):
    from tests.unit.delivery.stream.raw_infos import data

    response = StubResponse(data)
    config = StubConfig(rawconfig)
    session = StubSession(
        config=config,
        response=response,
        session_cxn_type=SessionCxnType.REFINITIV_DATA_AND_DEPLOYED,
    )
    provider = PlatformAndDeployedCxnConfigProvider()
    return provider, session


@pytest.mark.parametrize(
    "rawconfig,expected_url",
    [
        args(
            input={
                "apis.streaming.middle.url": "/head/middle/v1",
                "apis.streaming.middle.endpoints.tail.path": "/",
                "apis.streaming.middle.endpoints.tail.locations": [],
                "apis.streaming.middle.endpoints.tail.protocols": [],
            },
            expected="wss://ap-southeast-1-aws-1-lrg.optimized-pricing:443/api.refinitiv.net",
        ),
        args(
            input={
                "apis.streaming.middle.url": "/head/middle/v1",
                "apis.streaming.middle.endpoints.tail.path": "/",
                "apis.streaming.middle.endpoints.tail.locations": "amer",
                "apis.streaming.middle.endpoints.tail.protocols": "OMM",
            },
            expected="wss://ap-southeast-1-aws-1-lrg.optimized-pricing:443/api.refinitiv.net",
        ),
        args(
            input={
                "apis.streaming.middle.endpoints.tail.direct-url": "wss://amer-3-t3.beta.streaming-pricing-api.refinitiv.com:443",
                "apis.streaming.middle.endpoints.tail.protocols": [],
            },
            expected="wss://amer-3-t3.beta.streaming-pricing-api.refinitiv.com:443/WebSocket",
        ),
        args(
            input={
                "apis.streaming.middle.endpoints.tail.direct-url": "//amer-3-t3.beta.streaming-pricing-api.refinitiv.com:443",
                "apis.streaming.middle.endpoints.tail.protocols": [],
            },
            expected="wss://amer-3-t3.beta.streaming-pricing-api.refinitiv.com:443/WebSocket",
        ),
        args(
            input={
                "apis.streaming.middle.endpoints.tail.direct-url": "//amer-3-t3.beta.streaming-pricing-api.refinitiv.com",
                "apis.streaming.middle.endpoints.tail.protocols": [],
            },
            expected="ws://amer-3-t3.beta.streaming-pricing-api.refinitiv.com:80/WebSocket",
        ),
        args(
            input={
                "apis.streaming.middle.endpoints.tail.direct-url": "amer-3-t3.beta.streaming-pricing-api.refinitiv.com",
                "apis.streaming.middle.endpoints.tail.protocols": [],
            },
            expected="ws://amer-3-t3.beta.streaming-pricing-api.refinitiv.com:80/WebSocket",
        ),
    ],
)
def test_configuration_parsed_correct(rawconfig, expected_url):
    # given
    provider, session = create_platform_and_deployed_cxn_config_provider(rawconfig)

    # when
    config = provider.get_cfg(session, "apis.streaming.middle.endpoints.tail")

    # then
    assert config.url == expected_url


@pytest.mark.parametrize(
    "rawconfig,expected_url",
    [
        args(
            input={
                "sessions.platform.default.realtime-distribution-system.url": "wss://amer-3-t3.beta.streaming-pricing-api.refinitiv.com:443",
            },
            expected="wss://amer-3-t3.beta.streaming-pricing-api.refinitiv.com:443/WebSocket",
        ),
        args(
            input={
                "sessions.platform.default.realtime-distribution-system.url": "//amer-3-t3.beta.streaming-pricing-api.refinitiv.com:443",
            },
            expected="wss://amer-3-t3.beta.streaming-pricing-api.refinitiv.com:443/WebSocket",
        ),
        args(
            input={
                "sessions.platform.default.realtime-distribution-system.url": "//amer-3-t3.beta.streaming-pricing-api.refinitiv.com",
            },
            expected="ws://amer-3-t3.beta.streaming-pricing-api.refinitiv.com:80/WebSocket",
        ),
        args(
            input={
                "sessions.platform.default.realtime-distribution-system.url": "amer-3-t3.beta.streaming-pricing-api.refinitiv.com",
            },
            expected="ws://amer-3-t3.beta.streaming-pricing-api.refinitiv.com:80/WebSocket",
        ),
    ],
)
def test_configuration_parsed_correct_without_deployed_host_in_config(
    rawconfig, expected_url
):
    provider, session = create_platform_and_deployed_cxn_config_provider(rawconfig)

    # when
    config = provider.get_cfg(session, "apis.streaming.pricing.endpoints.main")

    # then
    assert config.url == expected_url


def test_will_raise_error_if_no_url_in_config():
    # given
    provider, session = create_platform_and_deployed_cxn_config_provider({})

    # then
    with pytest.raises(KeyError, match="apis.streaming.middle.url"):
        # when
        provider.get_cfg(session, "apis.streaming.middle.endpoints.tail")


@pytest.mark.parametrize(
    "deployed_host,expected_url",
    [
        args(
            input="wss://amer-3-t3.beta.streaming-pricing-api.refinitiv.com:443",
            expected="wss://amer-3-t3.beta.streaming-pricing-api.refinitiv.com:443/WebSocket",
        ),
        args(
            input="10.3.177.158:15000",
            expected="ws://10.3.177.158:15000/WebSocket",
        ),
        args(
            input="//amer-3-t3.beta.streaming-pricing-api.refinitiv.com:443",
            expected="wss://amer-3-t3.beta.streaming-pricing-api.refinitiv.com:443/WebSocket",
        ),
        args(
            input="amer-3-t3.beta.streaming-pricing-api.refinitiv.com:443",
            expected="wss://amer-3-t3.beta.streaming-pricing-api.refinitiv.com:443/WebSocket",
        ),
        args(
            input="//amer-3-t3.beta.streaming-pricing-api.refinitiv.com",
            expected="ws://amer-3-t3.beta.streaming-pricing-api.refinitiv.com:80/WebSocket",
        ),
        args(
            input="amer-3-t3.beta.streaming-pricing-api.refinitiv.com",
            expected="ws://amer-3-t3.beta.streaming-pricing-api.refinitiv.com:80/WebSocket",
        ),
    ],
)
def test_configuration_parsed_correct_with_deployed_host(deployed_host, expected_url):
    # given
    session = StubSession(
        deployed_platform_host=deployed_host,
        session_cxn_type=SessionCxnType.REFINITIV_DATA_AND_DEPLOYED,
    )
    provider = PlatformAndDeployedCxnConfigProvider()

    # when
    config = provider.get_cfg(session, "apis.streaming.pricing.endpoints.main")

    # then
    assert config.url == expected_url


def test_will_raise_error_if_no_path_in_config():
    # given
    provider, session = create_platform_and_deployed_cxn_config_provider(
        {
            "apis.streaming.middle.url": "/head/middle/v1",
        }
    )

    # then
    with pytest.raises(KeyError, match="Cannot find discovery endpoint"):
        # when
        provider.get_cfg(session, "apis.streaming.middle.endpoints.tail")


def test_will_raise_error_if_no_locations_in_config():
    # given
    provider, session = create_platform_and_deployed_cxn_config_provider(
        {
            "apis.streaming.middle.url": "/head/middle/v1",
            "apis.streaming.middle.endpoints.tail.path": "/",
        }
    )

    # then
    with pytest.raises(
        KeyError, match="apis.streaming.middle.endpoints.tail.locations"
    ):
        # when
        provider.get_cfg(session, "apis.streaming.middle.endpoints.tail")


def test_will_raise_error_if_no_protocols_in_config():
    # given
    provider, session = create_platform_and_deployed_cxn_config_provider(
        {
            "apis.streaming.middle.url": "/head/middle/v1",
            "apis.streaming.middle.endpoints.tail.path": "/",
            "apis.streaming.middle.endpoints.tail.locations": [],
        }
    )

    # then
    with pytest.raises(
        KeyError, match="apis.streaming.middle.endpoints.tail.protocols"
    ):
        # when
        provider.get_cfg(session, "apis.streaming.middle.endpoints.tail")


def test_infos_parsed_correct():
    # given
    provider, session = create_platform_and_deployed_cxn_config_provider(
        {
            "apis.streaming.middle.url": "http://example.dom/head/middle/v1",
            "apis.streaming.middle.endpoints.tail.path": "/",
            "apis.streaming.middle.endpoints.tail.locations": [],
            "apis.streaming.middle.endpoints.tail.protocols": [],
        }
    )

    # when
    config = provider.get_cfg(session, "apis.streaming.middle.endpoints.tail")

    # then
    assert len(config.urls) == 12


def test_infos_filtered_by_location_correct():
    # given
    expected_location = "ap-southeast-1a"
    provider, session = create_platform_and_deployed_cxn_config_provider(
        {
            "apis.streaming.middle.url": "path/to/head/middle/v1",
            "apis.streaming.middle.endpoints.tail.path": "/",
            "apis.streaming.middle.endpoints.tail.locations": [expected_location],
            "apis.streaming.middle.endpoints.tail.protocols": [],
        }
    )

    # when
    config = provider.get_cfg(session, "apis.streaming.middle.endpoints.tail")

    # then
    assert len(config.urls) == 2



__all__ = (
    "BermudanSwaptionDefinition",
    "BuySell",
    "CallPut",
    "Definition",
    "ExerciseScheduleType",
    "ExerciseStyle",
    "InputFlow",
    "PremiumSettlementType",
    "PriceSide",
    "PricingParameters",
    "SwaptionMarketDataRule",
    "SwaptionSettlementType",
    "SwaptionType",
)

from ._bermudan_swaption_definition import BermudanSwaptionDefinition
from ._definition import Definition
from ._enums import (
    BuySell,
    CallPut,
    ExerciseScheduleType,
    ExerciseStyle,
    PremiumSettlementType,
    PriceSide,
    SwaptionSettlementType,
    SwaptionType,
)
from ._swaption_market_data_rule import SwaptionMarketDataRule
from ._swaption_pricing_parameters import PricingParameters
from ._models import InputFlow



def check_document_title_contains_query(response, query):
    for search_item in response.data.hits:
        assert (
            query.lower() in search_item.DocumentTitle.lower()
        ), f"Found result which does not contain query: {search_item}"


def check_business_entity_for_every_document_in_response(
    response, expected_business_entity
):
    for hits_item in response.data.hits:
        assert (
            hits_item.BusinessEntity == expected_business_entity
        ), f"Found the document with BusinessEntity different from expected: {hits_item.BusinessEntit}"


def check_hits_and_dataframe_columns(response, hits_item_attributes):
    assert len(response.data.hits) == 10
    for attr in hits_item_attributes:
        assert hasattr(
            response.data.hits[0], attr
        ), f"{attr} is missing for HitsItem object"
        assert (
            attr in response.data.df.columns
        ), f"{attr} header is missing in DataFrame"


def check_navigator_entity_in_response(
    response,
    expected_navigator_entity,
    expected_sub_navigator=None,
    navigator_props=None,
    sub_navigator_props=None,
):
    def check_buckets_attrs(bucket, navigator_name):
        assert hasattr(
            bucket, "count"
        ), f"count attribute is missing for {navigator_name} bucket"
        assert hasattr(
            bucket, "label"
        ), f"label attribute is missing for {navigator_name} bucket"

    def check_navigator_props(bucket, props):
        for entity in props:
            assert bucket[entity] is not None, f"{entity} data is missing"

    assert (
        expected_navigator_entity in response.data.navigators
    ), f"{expected_navigator_entity} is missing in requested navigators"
    assert (
        response.data.navigators[expected_navigator_entity].name
        == expected_navigator_entity
    )

    if response.data.raw["Navigators"][expected_navigator_entity]["Buckets"]:
        navigator_buckets = response.data.navigators[expected_navigator_entity].buckets
        for bucket in navigator_buckets:
            check_buckets_attrs(bucket, expected_navigator_entity)

            if navigator_props:
                check_navigator_props(bucket, navigator_props)

            if expected_sub_navigator:
                assert (
                    bucket.navigator is not None
                ), f"navigator attribute is missing for {expected_navigator_entity} bucket"
                assert bucket.navigator.name == expected_sub_navigator

                for bucket_item in bucket.navigator.buckets:
                    check_buckets_attrs(bucket_item, expected_navigator_entity)
                    assert (
                        bucket_item.navigator is None
                    ), f"sub sub navigators are not expected"

                    if sub_navigator_props:
                        check_navigator_props(bucket_item, sub_navigator_props)



import allure
import pytest

from refinitiv.data.content.ownership.fund import shareholders_report
from tests.integration.constants_list import HttpStatusCode, HttpReason
from tests.integration.content.ownership.helpers import (
    check_response_dataframe_instrument_contains_expected_universes,
)
from tests.integration.helpers import (
    check_response_status,
    check_non_empty_response_data,
    check_response_dataframe_contains_columns_names,
)

EXPECTED_COLUMN_NAMES = [
    "instrument",
    "calcdate",
    "investorid",
    "investorpermid",
    "TR.FundPortfolioName",
    "TR.FundParentType",
    "TR.FundClassId",
    "TR.FundInvestorType",
    "TR.FundRegion",
    "TR.FundInvtStyleCode",
    "TR.FundInvtOrientation",
    "TR.FundTurnoverRating",
    "TR.FundOwnershipTurnover",
    "TR.FundTotalEquityAssets",
    "TR.FundAdjShrsHeld",
    "TR.FdAdjPctOfShrsOutHeld",
    "TR.FundPctPortfolio",
    "TR.FdAdjSharesHeldValue",
    "TR.FundHoldingsDate",
    "TR.FundFilingType",
    "TR.FdAdjShrsHeldChange",
    "TR.FdAdjShrsHeldValChg",
    "SharesOutstanding Change %",
    "TR.FdFundPctPositionChg",
    "TR.FdWeightCountry",
    "TR.FdPctCountry",
    "TR.FdWeightSector",
    "TR.FdPctSector",
    "TR.FdWeightExchange",
    "TR.FdPctExchange",
    "TR.PrevFundAdjSharesHeld",
    "TR.PrevAdjPctShrsOutHeld",
    "TR.PrevAdjSharesHeldValue",
    "TR.PrevFundHoldingsDate",
    "TR.PrevFundFilingType",
    "TR.FundContactFirstName",
    "TR.FundContactMiddleInit",
    "TR.FundContactLastName",
    "TR.FundContactTelCntry",
    "TR.FundContactTelAreaCode",
    "TR.FundContactTelNumber",
    "TR.FundContactTelExt",
    "TR.FundContactMetroArea",
    "TR.FundAddressCity",
    "TR.FundAddrCountry",
    "TR.FundParentLegacyId",
]


@allure.suite("Content object - Ownership Fund Shareholders Report")
@allure.feature("Content object - Ownership Fund Shareholders Report")
@allure.severity(allure.severity_level.CRITICAL)
class TestFundShareholdersReport:
    @allure.title(
        "Create Fund Shareholders Report definition object with valid params and get data asynchronously"
    )
    @pytest.mark.caseid("37478939")
    @pytest.mark.parametrize("universe,limit", [("TRI.N", 10)])
    async def test_create_fund_shareholders_report_async(
        self, open_platform_session_async, universe, limit
    ):
        response = await shareholders_report.Definition(
            universe, limit, use_field_names_in_headers=True
        ).get_data_async()
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)
        check_response_dataframe_contains_columns_names(response, EXPECTED_COLUMN_NAMES)
        check_response_dataframe_instrument_contains_expected_universes(
            response, universe
        )



import pytest

from refinitiv.data.delivery._stream._stream_cxn_config_data import (
    StreamServiceInfo,
    PlatformStreamCxnConfig,
)


def test_first_info():
    # given
    first_info = StreamServiceInfo("", "first_info", "", "", "", "", "")
    infos = [
        first_info,
        StreamServiceInfo("", "second_info", "", "", "", "", ""),
        StreamServiceInfo("", "third_info", "", "", "", "", ""),
    ]
    config = PlatformStreamCxnConfig(infos, protocols=[""])

    # when

    # then
    assert config.info == first_info


def test_next_available_info():
    # given
    second_info = StreamServiceInfo("", "second_info", "", "", "", "", "")
    infos = [
        StreamServiceInfo("", "first_info", "", "", "", "", ""),
        second_info,
        StreamServiceInfo("", "third_info", "", "", "", "", ""),
    ]
    config = PlatformStreamCxnConfig(infos, protocols=[""])

    # when
    config.next_available_info()

    # then
    assert config.info == second_info


def test_last_info():
    # given
    infos = [
        StreamServiceInfo("", "first_info", "", "", "", "", ""),
        StreamServiceInfo("", "second_info", "", "", "", "", ""),
        StreamServiceInfo("", "third_info", "", "", "", "", ""),
    ]
    config = PlatformStreamCxnConfig(infos, protocols=[""])
    config.next_available_info()
    config.next_available_info()

    # then
    with pytest.raises(StopIteration):
        # when
        config.next_available_info()


def test_has_available_info():
    # given
    infos = [
        StreamServiceInfo("", "first_info", "", "", "", "", ""),
        StreamServiceInfo("", "second_info", "", "", "", "", ""),
        StreamServiceInfo("", "third_info", "", "", "", "", ""),
    ]
    config = PlatformStreamCxnConfig(infos, protocols=[""])

    # then
    assert config.has_available_info() is True


def test_info_not_available():
    # given
    infos = [
        StreamServiceInfo("", "first_info", "", "", "", "", ""),
        StreamServiceInfo("", "second_info", "", "", "", "", ""),
        StreamServiceInfo("", "third_info", "", "", "", "", ""),
    ]
    config = PlatformStreamCxnConfig(infos, protocols=[""])

    # when
    config.info_not_available()

    # then
    assert config.has_available_info() is True


def test_all_infos_not_available():
    # given
    infos = [
        StreamServiceInfo("", "first_info", "", "", "", "", ""),
        StreamServiceInfo("", "second_info", "", "", "", "", ""),
        StreamServiceInfo("", "third_info", "", "", "", "", ""),
    ]
    config = PlatformStreamCxnConfig(infos, protocols=[""])

    # when
    config.info_not_available()

    config.next_available_info()
    config.info_not_available()

    config.next_available_info()
    config.info_not_available()

    # then
    assert config.has_available_info() is False


def test_raise_if_all_infos_not_available_and_call_next_available_info():
    # given
    infos = [
        StreamServiceInfo("", "first_info", "", "", "", "", ""),
        StreamServiceInfo("", "second_info", "", "", "", "", ""),
        StreamServiceInfo("", "third_info", "", "", "", "", ""),
    ]
    config = PlatformStreamCxnConfig(infos, protocols=[""])

    config.info_not_available()

    config.next_available_info()
    config.info_not_available()

    config.next_available_info()
    config.info_not_available()

    # then
    with pytest.raises(ValueError):
        # when
        config.next_available_info()



import pytest

from refinitiv.data._errors import RDError
from refinitiv.data.content.search import lookup

from tests.unit.conftest import StubSession, StubFailedResponse


def test_raise_error_when_not_valid_request():
    # given
    session = StubSession(is_open=True, response=StubFailedResponse())
    definition = lookup.Definition(view="invalid_value", terms="", scope="", select="")

    with pytest.raises(RDError):
        definition.get_data(session=session)



import allure
import pytest

from refinitiv.data._errors import ScopeError
from refinitiv.data.delivery import endpoint_request
from tests.integration.constants_list import HttpStatusCode, HttpReason
from tests.integration.delivery.endpoint_request.conftest import definition

from tests.integration.helpers import (
    get_async_response_from_definition,
    check_response_status,
    assert_error,
)


@allure.suite("Endpoint request")
@allure.feature("Endpoint request")
@allure.severity(allure.severity_level.CRITICAL)
class TestEndpointRequest:
    @allure.title(
        "Create endpoint request definition object with POST method, valid params - synchronously"
    )
    @pytest.mark.caseid("C37753624")
    @pytest.mark.parametrize(
        "url,body_param",
        [
            (
                "https://api.refinitiv.com/data/historical-pricing/v1/views/interday-summaries",
                {
                    "universe": ["IBM.N", "FB.O", "MSFT.O", "AAPL.O", "TSLA.O"],
                    "end": "2020-12-01",
                },
            )
        ],
    )
    @pytest.mark.smoke
    def test_endpoint_request_definition_object_with_post_method_valid_body_params_and_get_data(
        self, open_session, url, body_param
    ):
        response = endpoint_request.Definition(
            url=url,
            method=endpoint_request.RequestMethod.POST,
            body_parameters=body_param,
        ).get_data()
        assert response.data.raw, f"Empty response.data.raw received"
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)

    @allure.title(
        "Create endpoint request definition object with GET method, valid params - synchronously"
    )
    @pytest.mark.caseid("C37753623")
    @pytest.mark.parametrize(
        "url,request_method,path_param,query_param",
        [
            (
                "/data/historical-pricing/v1/views/interday-summaries/{universe}",
                "GET",
                {"universe": "VOD.L"},
                None,
            ),
            (
                "https://localhost:9000/data/news/v1/headlines",
                endpoint_request.RequestMethod.GET,
                None,
                {
                    "query": "Kyiv",
                    "count": "100",
                    "dateFrom": "2021-05-15",
                    "dateTo": "2022-03-19",
                    "sort": "newToOld",
                },
            ),
            (
                "/data/estimates/v1/view-actuals-kpi/interim",
                endpoint_request.RequestMethod.GET,
                None,
                {"universe": ["IBM", "MSFT"]},
            ),
        ],
    )
    @pytest.mark.smoke
    def test_endpoint_request_definition_object_with_get_method_valid_params_and_get_data(
        self,
        url,
        request_method,
        path_param,
        query_param,
        open_platform_session,
        open_platform_session_with_rdp_creds,
    ):
        definition = endpoint_request.Definition(
            url=url,
            method=request_method,
            path_parameters=path_param,
            query_parameters=query_param,
        )
        if "estimates" in url:
            session = open_platform_session_with_rdp_creds
            response = definition.get_data(session=session)
        else:
            response = definition.get_data()

        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        assert response.data.raw, f"Empty response.data.raw received"

    @allure.title(
        "Create endpoint request definition object with GET method, universe and query params in URL"
    )
    @pytest.mark.caseid("C37753626")
    @pytest.mark.parametrize(
        "url",
        [
            "/data/historical-pricing/v1/views/events/IBM.N?eventTypes=trade&count=1",
        ],
    )
    @pytest.mark.smoke
    def test_endpoint_request_definition_object_with_get_method_universe_and_params_in_url_and_get_data(
        self, open_session, url
    ):
        response = endpoint_request.Definition(
            url=url, method=endpoint_request.RequestMethod.GET
        ).get_data()
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        assert response.data.raw, f"Empty response.data.raw received"
        assert url in str(response.request_message.url)

    @allure.title("Create endpoint request definition object using closed session")
    @pytest.mark.caseid("C37753631")
    async def test_endpoint_request_definition_object_using_closed_session(
        self, open_desktop_session_async
    ):
        await open_desktop_session_async.close_async()

        with pytest.raises(ValueError) as error:
            await get_async_response_from_definition(definition)
        assert str(error.value) == "Session is not opened. Can't send any request"

    @allure.title("Create endpoint request definition object using invalid scope")
    @pytest.mark.caseid("")
    async def test_endpoint_request_definition_object_using_invalid_scope(
        self, open_platform_session_async
    ):
        with pytest.raises(ScopeError) as error:
            await get_async_response_from_definition(definition)
        assert "Missing scopes" in str(error.value)

    @allure.title(
        "Create endpoint request definition object without any params and get error"
    )
    @pytest.mark.caseid("C37753628")
    def test_endpoint_request_definition_object_without_any_params_and_get_error(self):
        with pytest.raises(TypeError) as error:
            endpoint_request.Definition().get_data()
        assert_error(error, "url")

    @allure.title(
        "Create endpoint request definition object with empty params and get error"
    )
    @pytest.mark.caseid("C37753629")
    @pytest.mark.parametrize(
        "url,path_param,expected_error",
        [
            ("", None, "Requested URL is missing, please provide valid URL"),
            (
                "/data/historical-pricing/v1/views/events/{universe}",
                None,
                "Path parameter 'universe' is missing, please provide path parameter",
            ),
        ],
    )
    def test_endpoint_request_definition_object_with_empty_params_and_get_error(
        self, open_session, url, path_param, expected_error
    ):
        with pytest.raises(ValueError) as error:
            endpoint_request.Definition(
                url=url,
                path_parameters=path_param,
            ).get_data()
        assert str(error.value) == expected_error



# coding: utf8

from typing import Optional, TYPE_CHECKING

from . import PricingParameters
from ._cds_definition import CdsInstrumentDefinition
from ._enums import (
    BusinessDayConvention,
    CdsConvention,
)
from ._models import (
    PremiumLegDefinition,
    ProtectionLegDefinition,
)
from .._base_definition import BaseDefinition
from ....._tools import try_copy_to_list

if TYPE_CHECKING:
    from ....._types import ExtendedParams, OptStrStrs


class Definition(BaseDefinition):
    """
    API endpoint for Financial Contract analytics,
    that returns calculations relevant to each contract type.

    Parameters
    ----------
    instrument_tag : str, optional
        User defined string to identify the instrument.It can be used to link output
        results to the instrument definition. Only alphabetic, numeric and '- _.#=@'
        characters are supported. Optional.
    instrument_code : str, optional
        A cds RIC that is used to retrieve the description of the cds contract.
        Optional. If null, the protection_leg and the premium_leg  must be provided.
    cds_convention : CdsConvention, optional
        Define the cds convention. Possible values are:
        - 'ISDA' (start_date will default to accrued_begin_date, end_date will be
          adjusted to IMM Date),
        - 'UserDefined' (start_date will default to step_in_date, end_date will not be
          adjusted). Optional. Defaults to 'ISDA'.
    trade_date : str, optional
        The date the cds contract was created. Optional. By default the valuation date.
    step_in_date : str, optional
        The effective protection date. Optional. By default the trade_date + 1 calendar.
    start_date : str, optional
        The date the cds starts accruing interest. Its effective date. Optional. By
        default it is the accrued_begin_date (the last IMM date before trade_date) if
        cds_convention is ISDA, else it is the step_in_date.
    end_date : str, optional
        The maturity date of the cds contract. Mandatory if instrument_code is null.
        Either the end_date or the tenor must be provided.
    tenor : str, optional
        The period code that represents the time between the start date and end date the
        contract. Mandatory if instrument_code is null. Either the end_date or the tenor
        must be provided.
    start_date_moving_convention : BusinessDayConvention, optional
        The method to adjust the start_date. The possible values are:
        - ModifiedFollowing (adjusts dates according to the Modified Following
          convention - next business day unless is it goes into the next month,
          preceeding is used in that  case),
        - NextBusinessDay (adjusts dates according to the Following convention - Next
          Business Day),
        - PreviousBusinessDay (adjusts dates  according to the Preceeding convention -
          Previous Business Day),
        - NoMoving (does not adjust dates),
        - BbswModifiedFollowing (adjusts dates  according to the BBSW Modified Following
          convention). Optional. By default 'NoMoving' is used.
    end_date_moving_convention : BusinessDayConvention, optional
        The method to adjust the end_date.. The possible values are:
        - ModifiedFollowing (adjusts dates according to the Modified Following
          convention - next business day unless is it goes into the next month,
          preceeding is used in that  case),
        - NextBusinessDay (adjusts dates according to the Following convention - Next
          Business Day),
        - PreviousBusinessDay (adjusts dates  according to the Preceeding convention -
          Previous Business Day),
        - NoMoving (does not adjust dates),
        - BbswModifiedFollowing (adjusts dates  according to the BBSW Modified Following
          convention). Optional. By default 'NoMoving' is used.
    adjust_to_isda_end_date : bool, optional
        The way the end_date is adjusted if computed from tenor input.    The possible
        values are:
        - true ( the end_date is an IMM date computed from start_date according to ISDA
          rules, ),
        - false ( the end_date is computed from start_date according to
          end_dateMovingConvention), Optional. By default true is used if cds_convention
          is ISDA, else false is used.
    protection_leg : ProtectionLegDefinition, optional
        The Protection Leg of the CDS. It is the default leg. Mandatory if instrumenCode
        is null. Optional if instrument_code not null.
    premium_leg : PremiumLegDefinition, optional
        The Premium Leg of the CDS. It is a swap leg paying a fixed coupon. Mandatory if
        instrument_code is null. Optional if instrument_code not null.
    accrued_begin_date : str, optional
        The last cashflow date. Optional. By default it is the last cashflow date.
    fields: list of str, optional
        Contains the list of Analytics that the quantitative analytic service will
        compute.
    pricing_parameters : PricingParameters, optional
        The pricing parameters to apply to this instrument. Optional. If pricing
        parameters are not provided at this level parameters defined globally at the
        request level are used. If no pricing parameters are provided globally default
        values apply.
    extended_params : dict, optional
        If necessary other parameters

    Methods
    -------
    get_data(session=session, on_response=on_response)
        Returns a response to the data platform
    get_stream(session=session)
        Get stream object of this definition

    Examples
    --------
    >>> import refinitiv.data.content.ipa.financial_contracts as rdf
    >>> definition = rdf.cds.Definition(
    ...     instrument_tag="Cds1_InstrumentCode",
    ...     instrument_code="BNPP5YEUAM=R",
    ...     cds_convention=rdf.cds.CdsConvention.ISDA,
    ...     end_date_moving_convention=rdf.cds.BusinessDayConvention.NO_MOVING,
    ...     adjust_to_isda_end_date=True,
    ...     pricing_parameters=rdf.cds.PricingParameters(
    ...         market_data_date="2020-01-01"
    ...     ),
    ...     fields=[
    ...         "InstrumentTag",
    ...         "ValuationDate",
    ...         "InstrumentDescription",
    ...         "StartDate",
    ...         "EndDate",
    ...         "SettlementDate",
    ...         "UpfrontAmountInDealCcy",
    ...         "CashAmountInDealCcy",
    ...         "AccruedAmountInDealCcy",
    ...         "AccruedBeginDate",
    ...         "NextCouponDate",
    ...         "UpfrontPercent",
    ...         "ConventionalSpreadBp",
    ...         "ParSpreadBp",
    ...         "AccruedDays",
    ...         "ErrorCode",
    ...         "ErrorMessage",
    ...     ],
    ... )
    >>> response = definition.get_data()
    >>> df = response.data.df
    """

    def __init__(
        self,
        instrument_tag: Optional[str] = None,
        instrument_code: Optional[str] = None,
        cds_convention: Optional[CdsConvention] = None,
        trade_date: Optional[str] = None,
        step_in_date: Optional[str] = None,
        start_date: Optional[str] = None,
        end_date: Optional[str] = None,
        tenor: Optional[str] = None,
        start_date_moving_convention: Optional[BusinessDayConvention] = None,
        end_date_moving_convention: Optional[BusinessDayConvention] = None,
        adjust_to_isda_end_date: Optional[bool] = None,
        protection_leg: Optional[ProtectionLegDefinition] = None,
        premium_leg: Optional[PremiumLegDefinition] = None,
        accrued_begin_date: Optional[str] = None,
        fields: "OptStrStrs" = None,
        pricing_parameters: Optional[PricingParameters] = None,
        extended_params: "ExtendedParams" = None,
    ):
        fields = try_copy_to_list(fields)
        definition = CdsInstrumentDefinition(
            cds_convention=cds_convention,
            end_date_moving_convention=end_date_moving_convention,
            premium_leg=premium_leg,
            protection_leg=protection_leg,
            start_date_moving_convention=start_date_moving_convention,
            accrued_begin_date=accrued_begin_date,
            adjust_to_isda_end_date=adjust_to_isda_end_date,
            end_date=end_date,
            instrument_code=instrument_code,
            instrument_tag=instrument_tag,
            start_date=start_date,
            step_in_date=step_in_date,
            tenor=tenor,
            trade_date=trade_date,
        )
        super().__init__(
            definition=definition,
            fields=fields,
            pricing_parameters=pricing_parameters,
            extended_params=extended_params,
        )



from inspect import signature

import pytest

import refinitiv.data.content.ipa.financial_contracts as rdf
from refinitiv.data.delivery._data._data_provider_factory import make_provider
from tests.unit.conftest import StubSession
from tests.unit.conftest import (
    remove_private_attributes,
    remove_dunder_methods,
    has_property_names_in_class,
    get_property_names,
)


def test_ipa_term_deposit_definition_attributes():
    expected_attributes = ["get_data", "get_data_async", "get_stream"]
    testing_attributes = dir(rdf.term_deposit.Definition)
    testing_attributes = remove_dunder_methods(testing_attributes)
    testing_attributes = remove_private_attributes(testing_attributes)
    assert expected_attributes == testing_attributes


def test_ipa_term_deposit_attributes():
    expected_attributes = [
        "BusinessDayConvention",
        "DateRollingConvention",
        "DayCountBasis",
        "Definition",
        "PriceSide",
        "PricingParameters",
    ]
    testing_attributes = dir(rdf.term_deposit)
    testing_attributes = remove_dunder_methods(testing_attributes)
    testing_attributes = remove_private_attributes(testing_attributes)
    assert expected_attributes == testing_attributes, testing_attributes


@pytest.mark.parametrize(
    argnames="input_data",
    ids=["Definition", "PricingParameters"],
    argvalues=[
        (
            rdf.term_deposit._term_deposit_definition.TermDepositInstrumentDefinition,
            {
                "instrument_tag": "instrument_tag",
                "instrument_code": "instrument_code",
                "start_date": "start_date",
                "end_date": "end_date",
                "tenor": "tenor",
                "notional_ccy": "notional_ccy",
                "notional_amount": 7.7,
                "fixed_rate_percent": 8.8,
                "payment_business_day_convention": rdf.term_deposit.BusinessDayConvention.NO_MOVING,
                "payment_roll_convention": rdf.term_deposit.DateRollingConvention.LAST28,
                "year_basis": rdf.term_deposit.DayCountBasis.DCB_30_360_ISDA,
                "calendar": "calendar",
            },
        ),
        (
            rdf.term_deposit.PricingParameters,
            {
                "price_side": rdf.term_deposit.PriceSide.LAST,
                "income_tax_percent": 2.2,
                "valuation_date": "valuation_date",
                "market_data_date": "market_data_date",
                "report_ccy": "report_ccy",
            },
        ),
    ],
)
def test_parameter(input_data):
    cls, kwargs = input_data
    args_names = list(kwargs.keys())
    inst = cls(**kwargs)

    s = signature(cls.__init__)
    # +1 for (self)
    assert len(s.parameters) == (
        len(args_names) + 1
    ), f"csl={cls}{str(set(get_property_names(cls)) - set(args_names))}"

    assert has_property_names_in_class(cls, args_names), set(args_names) - set(
        get_property_names(cls)
    )

    for k, v in kwargs.items():
        attr = getattr(inst, k)
        assert attr == v, k


def test_request_body():
    # given
    expected_body = {
        "fields": ["1", "2"],
        "universe": [
            {
                "instrumentDefinition": {
                    "startDate": "start_date",
                },
                "extended_params": "extended_params",
                "instrumentType": "TermDeposit",
                "pricingParameters": {"marketDataDate": "market_data_date"},
            }
        ],
    }
    definition = rdf.term_deposit.Definition(
        start_date="start_date",
        fields=["1", "2"],
        pricing_parameters=rdf.term_deposit.PricingParameters(
            market_data_date="market_data_date"
        ),
        extended_params={"extended_params": "extended_params"},
    )

    # when
    content_type = definition._kwargs["__content_type__"]
    provider = make_provider(content_type)
    request = provider.request.create(StubSession(), "url", **definition._kwargs)

    # then
    testing_body = request.json
    assert testing_body == expected_body, expected_body
